@@ -13,10 +13,11 @@
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
+#include "libcommon.h"
 
 #define BLOCK_SIZE 128
 
-ssize_t getdelims(char **lineptr, size_t *n, const char *delims, FILE *stream) {
+static ssize_t getdelims(char **lineptr, size_t *n, const char *delims, FILE *stream) {
   void *tmp;
   int c;
   size_t i;
@@ -40,9 +41,9 @@ ssize_t getdelims(char **lineptr, size_t *n, const char *delims, FILE *stream) {
     (*lineptr)[i++] = (unsigned char)c;
     if (index(delims, c)) break;
   }
-  (*lineptr)[i] = '\0';
+  if (i != 0) (*lineptr)[i] = '\0';
 
-  return (c == EOF) ? -1 : (ssize_t)i;
+  return (i == 0) ? -1 : (ssize_t)i;
 }
 
 
