@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: lirc.c,v 1.3 2004/12/20 17:30:05 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * lirc-specific stuff
  *
@@ -27,8 +25,6 @@
 #include "config.h"
 #endif
 
-#ifdef HAVE_LIRC
-
 #include <pthread.h>
 #include <signal.h>
 
@@ -94,5 +90,3 @@ int fbxine_init_lirc(void)
 	pthread_create(&fbxine.lirc.thread, 0, lirc_loop, 0);
 	return 1;
 }
-
-#endif /* HAVE_LIRC */
