@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: control.c,v 1.111 2005/04/13 19:45:08 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  */
@@ -33,15 +31,14 @@
 #include <X11/keysym.h>
 #include <pthread.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
 #include "common.h"
 
-#define TEST_VO_VALUE(val)  (val < 65535/3 || val > 65535*2/3) ? (65535 - val) : 0
+#define CONTROL_MIN     0
+#define CONTROL_MAX     65535
+#define CONTROL_STEP    565	/* approx. 1 pixel slider step */
+
+#define TEST_VO_VALUE(val)  (val < CONTROL_MAX/3 || val > CONTROL_MAX*2/3) ? (CONTROL_MAX - val) : 0
 
-extern gGui_t          *gGui;
 
 typedef struct {
   Window                window;
@@ -69,16 +66,20 @@ static int hue_ena, sat_ena, bright_ena, contr_ena;
 
 
 static void hue_changes_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui->video_settings.hue = cfg->num_value;
+  gGui->video_settings.hue =
+    (cfg->num_value < 0) ? gGui->video_settings.default_hue : cfg->num_value;
 }
 static void brightness_changes_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui->video_settings.brightness = cfg->num_value;
+  gGui->video_settings.brightness =
+    (cfg->num_value < 0) ? gGui->video_settings.default_brightness : cfg->num_value;
 }
 static void saturation_changes_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui->video_settings.saturation = cfg->num_value;
+  gGui->video_settings.saturation =
+    (cfg->num_value < 0) ? gGui->video_settings.default_saturation : cfg->num_value;
 }
 static void contrast_changes_cb(void *data, xine_cfg_entry_t *cfg) {
-  gGui->video_settings.contrast = cfg->num_value;
+  gGui->video_settings.contrast =
+    (cfg->num_value < 0) ? gGui->video_settings.default_contrast : cfg->num_value;
 }
 
 /*
@@ -268,31 +269,23 @@ static void probe_active_controls(void) {
 void control_reset(void) {
   
   if(hue_ena) {
-    set_current_param(XINE_PARAM_VO_HUE,
-		      (xitk_slider_get_min(control->hue) +
-		       xitk_slider_get_max(control->hue)) / 2);
-    config_update_num("gui.vo_hue", get_current_param(XINE_PARAM_VO_HUE));
+    set_current_param(XINE_PARAM_VO_HUE, gGui->video_settings.default_hue);
+    config_update_num("gui.vo_hue", -1);
   }
 
   if(sat_ena) {
-    set_current_param(XINE_PARAM_VO_SATURATION,
-		      (xitk_slider_get_min(control->sat) +
-		       xitk_slider_get_max(control->sat)) / 2);
-    config_update_num("gui.vo_saturation", get_current_param(XINE_PARAM_VO_SATURATION));
+    set_current_param(XINE_PARAM_VO_SATURATION, gGui->video_settings.default_saturation);
+    config_update_num("gui.vo_saturation", -1);
   }
 
   if(bright_ena) {
-    set_current_param(XINE_PARAM_VO_BRIGHTNESS,
-		      (xitk_slider_get_min(control->bright) +
-		       xitk_slider_get_max(control->bright)) / 2);
-    config_update_num("gui.vo_brightness", get_current_param(XINE_PARAM_VO_BRIGHTNESS));
+    set_current_param(XINE_PARAM_VO_BRIGHTNESS, gGui->video_settings.default_brightness);
+    config_update_num("gui.vo_brightness", -1);
   }
 
   if(contr_ena) {
-    set_current_param(XINE_PARAM_VO_CONTRAST,
-		      (xitk_slider_get_min(control->contr) +
-		       xitk_slider_get_max(control->contr)) / 2);
-    config_update_num("gui.vo_contrast", get_current_param(XINE_PARAM_VO_CONTRAST));
+    set_current_param(XINE_PARAM_VO_CONTRAST, gGui->video_settings.default_contrast);
+    config_update_num("gui.vo_contrast", -1);
   }
 
   update_sliders_video_settings();
@@ -304,66 +297,78 @@ void control_config_register(void) {
   
   if(hue_ena) {
     gGui->video_settings.hue = 
-     xine_config_register_range(gGui->xine, "gui.vo_hue",
+     xine_config_register_range(__xineui_global_xine_instance, "gui.vo_hue",
 				-1,
-				0, 65535,
-				_("hue value"),
-				_("Hue value."), 
+				CONTROL_MIN, CONTROL_MAX,
+				CONFIG_NO_DESC, /* _("hue value"), */
+				CONFIG_NO_HELP, /* _("Hue value."), */
 				CONFIG_LEVEL_DEB,
 				hue_changes_cb, 
 				CONFIG_NO_DATA);
-    if( gGui->video_settings.hue < 0 )
+    gGui->video_settings.default_hue = get_current_param(XINE_PARAM_VO_HUE);
+    if(gGui->video_settings.hue < 0)
+      gGui->video_settings.hue = gGui->video_settings.default_hue;
+    else {
+      set_current_param(XINE_PARAM_VO_HUE, gGui->video_settings.hue);
       gGui->video_settings.hue = get_current_param(XINE_PARAM_VO_HUE);
-    else 
-      set_current_param(XINE_PARAM_VO_HUE,gGui->video_settings.hue);
+    }
   }
   
   if(bright_ena) {
     gGui->video_settings.brightness = 
-     xine_config_register_range(gGui->xine, "gui.vo_brightness",
+     xine_config_register_range(__xineui_global_xine_instance, "gui.vo_brightness",
 				-1,
-				0, 65535,
-				_("brightness value"),
-				_("Brightness value."), 
+				CONTROL_MIN, CONTROL_MAX,
+				CONFIG_NO_DESC, /* _("brightness value"), */
+				CONFIG_NO_HELP, /* _("Brightness value."), */
 				CONFIG_LEVEL_DEB,
 				brightness_changes_cb, 
 				CONFIG_NO_DATA);
-    if( gGui->video_settings.brightness < 0 )
+    gGui->video_settings.default_brightness = get_current_param(XINE_PARAM_VO_BRIGHTNESS);
+    if(gGui->video_settings.brightness < 0)
+      gGui->video_settings.brightness = gGui->video_settings.default_brightness;
+    else {
+      set_current_param(XINE_PARAM_VO_BRIGHTNESS, gGui->video_settings.brightness);
       gGui->video_settings.brightness = get_current_param(XINE_PARAM_VO_BRIGHTNESS);
-    else 
-      set_current_param(XINE_PARAM_VO_BRIGHTNESS,gGui->video_settings.brightness);
+    }
   }
 
   if(sat_ena) {
    gGui->video_settings.saturation = 
-     xine_config_register_range(gGui->xine, "gui.vo_saturation",
+     xine_config_register_range(__xineui_global_xine_instance, "gui.vo_saturation",
 				-1,
-				0, 65535,
-				_("saturation value"),
-				_("Saturation value."), 
+				CONTROL_MIN, CONTROL_MAX,
+				CONFIG_NO_DESC, /* _("saturation value"), */
+				CONFIG_NO_HELP, /* _("Saturation value."), */
 				CONFIG_LEVEL_DEB,
 				saturation_changes_cb, 
 				CONFIG_NO_DATA);
-    if( gGui->video_settings.saturation < 0 )
+    gGui->video_settings.default_saturation = get_current_param(XINE_PARAM_VO_SATURATION);
+    if(gGui->video_settings.saturation < 0)
+      gGui->video_settings.saturation = gGui->video_settings.default_saturation;
+    else {
+      set_current_param(XINE_PARAM_VO_SATURATION, gGui->video_settings.saturation);
       gGui->video_settings.saturation = get_current_param(XINE_PARAM_VO_SATURATION);
-    else 
-      set_current_param(XINE_PARAM_VO_SATURATION,gGui->video_settings.saturation);
+    }
   }
 
   if(contr_ena) {
     gGui->video_settings.contrast = 
-     xine_config_register_range(gGui->xine, "gui.vo_contrast",
+     xine_config_register_range(__xineui_global_xine_instance, "gui.vo_contrast",
 				-1,
-				0, 65535,
-				_("contrast value"),
-				_("Contrast value."), 
+				CONTROL_MIN, CONTROL_MAX,
+				CONFIG_NO_DESC, /* _("contrast value"), */
+				CONFIG_NO_HELP, /* _("Contrast value."), */
 				CONFIG_LEVEL_DEB,
 				contrast_changes_cb, 
 				CONFIG_NO_DATA);
-    if( gGui->video_settings.contrast < 0 )
+    gGui->video_settings.default_contrast = get_current_param(XINE_PARAM_VO_CONTRAST);
+    if(gGui->video_settings.contrast < 0)
+      gGui->video_settings.contrast = gGui->video_settings.default_contrast;
+    else {
+      set_current_param(XINE_PARAM_VO_CONTRAST, gGui->video_settings.contrast);
       gGui->video_settings.contrast = get_current_param(XINE_PARAM_VO_CONTRAST);
-    else 
-      set_current_param(XINE_PARAM_VO_CONTRAST,gGui->video_settings.contrast);
+    }
   }
 }
 
@@ -405,7 +410,7 @@ void control_exit(xitk_widget_t *w, void *data) {
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(control->widget_list)));
     XUnlockDisplay(gGui->display);
 
-    free(control->widget_list);
+    XITK_WIDGET_LIST_FREE(control->widget_list);
 
     for(i = 0; i < control->skins_num; i++)
       free((char *)control->skins[i]);
@@ -580,7 +585,7 @@ void control_panel(void) {
   XSizeHints                 hint;
   XWMHints                  *wm_hint;
   XSetWindowAttributes       attr;
-  char                      *title;
+  char                      *title = _("xine Control Window");
   Atom                       prop;
   MWMHints                   mwmhints;
   XClassHint                *xclasshint;
@@ -591,15 +596,13 @@ void control_panel(void) {
   xitk_combo_widget_t        cmb;
   xitk_widget_t             *w;
 
-  xine_strdupa(title, _("xine Control Window"));
-
   XITK_WIDGET_INIT(&br, gGui->imlib_data);
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
   XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
   XITK_WIDGET_INIT(&sl, gGui->imlib_data);
   XITK_WIDGET_INIT(&cmb, gGui->imlib_data);
 
-  control = (_control_t *) xine_xmalloc(sizeof(_control_t));
+  control = (_control_t *) calloc(1, sizeof(_control_t));
 
   XLockDisplay(gGui->display);
   
@@ -612,14 +615,14 @@ void control_panel(void) {
 
   XUnlockDisplay(gGui->display);
 
-  hint.x = xine_config_register_num (gGui->xine, "gui.control_x", 
+  hint.x = xine_config_register_num (__xineui_global_xine_instance, "gui.control_x", 
 				     200,
 				     CONFIG_NO_DESC,
 				     CONFIG_NO_HELP,
 				     CONFIG_LEVEL_DEB,
 				     CONFIG_NO_CB,
 				     CONFIG_NO_DATA);
-  hint.y = xine_config_register_num (gGui->xine, "gui.control_y", 
+  hint.y = xine_config_register_num (__xineui_global_xine_instance, "gui.control_y", 
 				     100,
 				     CONFIG_NO_DESC,
 				     CONFIG_NO_HELP,
@@ -654,7 +657,7 @@ void control_panel(void) {
 				   CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 				   &attr);
   
-  XmbSetWMProperties(gGui->display, control->window, title, title, NULL, 0, 
+  XmbSetWMProperties(gGui->display, control->window, _(title), _(title), NULL, 0, 
                      &hint, NULL, NULL);
   
   XSelectInput(gGui->display, control->window, INPUT_MOTION | KeymapStateMask);
@@ -716,7 +719,7 @@ void control_panel(void) {
   xitk_widget_list_set(control->widget_list, WIDGET_LIST_GC, gc);
   
   { /* All of sliders are disabled by default*/
-    int min = 0, max = 65535, step = 565, cur;
+    int cur;
 
     lbl.window = XITK_WIDGET_LIST_WINDOW(control->widget_list);
     lbl.gc     = XITK_WIDGET_LIST_GC(control->widget_list);
@@ -725,9 +728,9 @@ void control_panel(void) {
     cur = get_current_param(XINE_PARAM_VO_HUE);
     
     sl.skin_element_name = "SliderCtlHue";
-    sl.min               = min;
-    sl.max               = max;
-    sl.step              = step;
+    sl.min               = CONTROL_MIN;
+    sl.max               = CONTROL_MAX;
+    sl.step              = CONTROL_STEP;
     sl.callback          = set_hue;
     sl.userdata          = NULL;
     sl.motion_callback   = set_hue;
@@ -748,9 +751,9 @@ void control_panel(void) {
     cur = get_current_param(XINE_PARAM_VO_SATURATION);
 
     sl.skin_element_name = "SliderCtlSat";
-    sl.min               = min;
-    sl.max               = max;
-    sl.step              = step;
+    sl.min               = CONTROL_MIN;
+    sl.max               = CONTROL_MAX;
+    sl.step              = CONTROL_STEP;
     sl.callback          = set_saturation;
     sl.userdata          = NULL;
     sl.motion_callback   = set_saturation;
@@ -771,9 +774,9 @@ void control_panel(void) {
     cur = get_current_param(XINE_PARAM_VO_BRIGHTNESS);
 
     sl.skin_element_name = "SliderCtlBright";
-    sl.min               = min;
-    sl.max               = max;
-    sl.step              = step;
+    sl.min               = CONTROL_MIN;
+    sl.max               = CONTROL_MAX;
+    sl.step              = CONTROL_STEP;
     sl.callback          = set_brightness;
     sl.userdata          = NULL;
     sl.motion_callback   = set_brightness;
@@ -794,9 +797,9 @@ void control_panel(void) {
     cur = get_current_param(XINE_PARAM_VO_CONTRAST);
 
     sl.skin_element_name = "SliderCtlCont";
-    sl.min               = min;
-    sl.max               = max;
-    sl.step              = step;
+    sl.min               = CONTROL_MIN;
+    sl.max               = CONTROL_MAX;
+    sl.step              = CONTROL_STEP;
     sl.callback          = set_contrast;
     sl.userdata          = NULL;
     sl.motion_callback   = set_contrast;
