@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: control.h,v 1.15 2004/04/13 19:55:47 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  */
 
@@ -26,15 +24,17 @@
 
 void control_config_register(void);
 
+void control_reset(void);
 void control_deinit(void);
 void control_panel(void);
-void control_change_skins(void);
+void control_change_skins(int);
 void control_exit(xitk_widget_t *, void *);
 int control_is_visible(void);
 int control_is_running(void);
 void control_toggle_visibility(xitk_widget_t *, void *);
 void control_raise_window(void);
 void control_show_tips(int enabled, unsigned long timeout);
+void control_update_tips_timeout(unsigned long timeout);
 void control_set_image_prop(int prop, int value);
 void control_reparent(void);
 
