@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2007 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: event_sender.c,v 1.32 2007/03/29 23:51:34 fsck-p Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  */
@@ -41,9 +39,6 @@
 #define WINDOW_WIDTH    250
 #define WINDOW_HEIGHT   221
 
-extern gGui_t          *gGui;
-
-static char            *eventerfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
 typedef struct {
   xitk_window_t        *xwin;
@@ -164,7 +159,7 @@ static void event_sender_select(xitk_widget_t *w, void *data) {
 }
 
 static void event_sender_menu(xitk_widget_t *w, void *data) {
-  int event     = (int) data;
+  int event = (int)(intptr_t) data;
   int events[7] = {
     XINE_EVENT_INPUT_MENU1, XINE_EVENT_INPUT_MENU2, XINE_EVENT_INPUT_MENU3,
     XINE_EVENT_INPUT_MENU4, XINE_EVENT_INPUT_MENU5, XINE_EVENT_INPUT_MENU6,
@@ -175,7 +170,7 @@ static void event_sender_menu(xitk_widget_t *w, void *data) {
 }
 
 static void event_sender_num(xitk_widget_t *w, void *data) {
-  int event      = (int) data;
+  int event = (int)(intptr_t) data;
   int events[11] = { 
     XINE_EVENT_INPUT_NUMBER_0, XINE_EVENT_INPUT_NUMBER_1, XINE_EVENT_INPUT_NUMBER_2,
     XINE_EVENT_INPUT_NUMBER_3, XINE_EVENT_INPUT_NUMBER_4, XINE_EVENT_INPUT_NUMBER_5,
@@ -187,7 +182,7 @@ static void event_sender_num(xitk_widget_t *w, void *data) {
 }
 
 static void event_sender_angle(xitk_widget_t *w, void *data) {
-  int event      = (int) data;
+  int event = (int)(intptr_t) data;
   int events[11] = { 
     XINE_EVENT_INPUT_ANGLE_NEXT, XINE_EVENT_INPUT_ANGLE_PREVIOUS
   };
@@ -338,7 +333,7 @@ static void event_sender_exit(xitk_widget_t *w, void *data) {
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(eventer->widget_list)));
     XUnlockDisplay(gGui->display);
     
-    free(eventer->widget_list);
+    XITK_WIDGET_LIST_FREE(eventer->widget_list);
     
     free(eventer);
     eventer = NULL;
@@ -406,16 +401,16 @@ void event_sender_panel(void) {
   int                        x, y, i;
   xitk_widget_t             *w;
 
-  eventer = (_eventer_t *) xine_xmalloc(sizeof(_eventer_t));
+  eventer = (_eventer_t *) calloc(1, sizeof(_eventer_t));
   
-  eventer->x = xine_config_register_num (gGui->xine, "gui.eventer_x", 
+  eventer->x = xine_config_register_num (__xineui_global_xine_instance, "gui.eventer_x", 
 					 80,
 					 CONFIG_NO_DESC,
 					 CONFIG_NO_HELP,
 					 CONFIG_LEVEL_DEB,
 					 CONFIG_NO_CB,
 					 CONFIG_NO_DATA);
-  eventer->y = xine_config_register_num (gGui->xine, "gui.eventer_y",
+  eventer->y = xine_config_register_num (__xineui_global_xine_instance, "gui.eventer_y",
 					 80,
 					 CONFIG_NO_DESC,
 					 CONFIG_NO_HELP,
@@ -464,7 +459,7 @@ void event_sender_panel(void) {
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 70, 40,
 					   "Black", "Black", "White", 
-					   eventerfontname)));
+					   hboldfontname)));
   xitk_enable_and_show_widget(eventer->navigator.up);
 
   x -= 70;
@@ -482,7 +477,7 @@ void event_sender_panel(void) {
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 70, 40,
 					   "Black", "Black", "White", 
-					   eventerfontname)));
+					   hboldfontname)));
   xitk_enable_and_show_widget(eventer->navigator.left);
 
   x += 75;
@@ -500,7 +495,7 @@ void event_sender_panel(void) {
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 60, 30,
 					   "Black", "Black", "White", 
-					   eventerfontname)));
+					   hboldfontname)));
   xitk_enable_and_show_widget(eventer->navigator.select);
 
   x += 65;
@@ -518,7 +513,7 @@ void event_sender_panel(void) {
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 70, 40,
 					   "Black", "Black", "White", 
-					   eventerfontname)));
+					   hboldfontname)));
   xitk_enable_and_show_widget(eventer->navigator.right);
 
   x -= 70;
@@ -536,7 +531,7 @@ void event_sender_panel(void) {
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 70, 40,
 					   "Black", "Black", "White", 
-					   eventerfontname)));
+					   hboldfontname)));
   xitk_enable_and_show_widget(eventer->navigator.down);
 
   x = 23 * 2 + 5 + (80 - 23 * 3) / 2 + 1; /* (+1 to round up) */
@@ -552,7 +547,7 @@ void event_sender_panel(void) {
     lb.align             = ALIGN_CENTER;
     lb.callback          = event_sender_num;
     lb.state_callback    = NULL;
-    lb.userdata          = (void *)i;
+    lb.userdata          = (void *)(intptr_t)i;
     lb.skin_element_name = NULL;
 
     if(!i)
@@ -562,7 +557,7 @@ void event_sender_panel(void) {
 	     (eventer->numbers.number[i] = 
 	      xitk_noskin_labelbutton_create(eventer->widget_list, 
 					     &lb, x, y, 23, 23,
-					     "Black", "Black", "White", eventerfontname)));
+					     "Black", "Black", "White", hboldfontname)));
     xitk_enable_and_show_widget(eventer->numbers.number[i]);
   
     if(!((i - 1) % 3)) {
@@ -588,14 +583,14 @@ void event_sender_panel(void) {
     lb.align             = ALIGN_CENTER;
     lb.callback          = event_sender_num;
     lb.state_callback    = NULL;
-    lb.userdata          = (void *)i;
+    lb.userdata          = (void *)(intptr_t)i;
     lb.skin_element_name = NULL;
     
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
 	     (eventer->numbers.number[i] = 
 	      xitk_noskin_labelbutton_create(eventer->widget_list, 
 					     &lb, x, y, 46, 23,
-					     "Black", "Black", "White", eventerfontname)));
+					     "Black", "Black", "White", hboldfontname)));
     xitk_enable_and_show_widget(eventer->numbers.number[i]);
   }
 
@@ -613,7 +608,7 @@ void event_sender_panel(void) {
 	   (eventer->angles.next = 
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 80, 23,
-					   "Black", "Black", "White", eventerfontname)));
+					   "Black", "Black", "White", hboldfontname)));
   xitk_enable_and_show_widget(eventer->angles.next);
 
   y += 23;
@@ -629,7 +624,7 @@ void event_sender_panel(void) {
 	   (eventer->angles.prev = 
 	    xitk_noskin_labelbutton_create(eventer->widget_list, 
 					   &lb, x, y, 80, 23,
-					   "Black", "Black", "White", eventerfontname)));
+					   "Black", "Black", "White", hboldfontname)));
   xitk_enable_and_show_widget(eventer->angles.prev);
 
 
@@ -659,13 +654,13 @@ void event_sender_panel(void) {
     lb.align             = ALIGN_CENTER;
     lb.callback          = event_sender_menu; 
     lb.state_callback    = NULL;
-    lb.userdata          = (void *)i;
+    lb.userdata          = (void *)(intptr_t)i;
     lb.skin_element_name = NULL;
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
 	     (eventer->menus.menu[i] = 
 	      xitk_noskin_labelbutton_create(eventer->widget_list, 
 					     &lb, x, y, 80, 23,
-					     "Black", "Black", "White", eventerfontname)));
+					     "Black", "Black", "White", hboldfontname)));
     xitk_enable_and_show_widget(eventer->menus.menu[i]);
 
     if(i == 2) {
@@ -694,7 +689,7 @@ void event_sender_panel(void) {
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(eventer->widget_list)), 
    (w = xitk_noskin_labelbutton_create(eventer->widget_list, 
 				       &lb, x, y, 70, 23,
-				       "Black", "Black", "White", eventerfontname)));
+				       "Black", "Black", "White", hboldfontname)));
   xitk_enable_and_show_widget(w);
   event_sender_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
 
