@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2007 the xine project
+ * Copyright (C) 2000-2008 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: main.c,v 1.332 2007/04/23 22:48:20 fsck-p Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * xine main for X11
  *
@@ -108,7 +106,7 @@ typedef struct {
 
 
 /* options args */
-static const char *short_options = "?hHgfvn"
+static const char short_options[] = "?hHgfvn"
 #ifdef HAVE_XINERAMA
  "F"
 #endif
@@ -120,7 +118,7 @@ static const char *short_options = "?hHgfvn"
 #endif
  "u:a:V:A:p::s:RW:G:BN:P:l::S:ZD::r:c:ET:I";
 
-static struct option long_options[] = {
+static const struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
 #ifdef HAVE_LIRC
   {"no-lirc"        , no_argument      , 0, 'L'                      },
@@ -246,13 +244,13 @@ static char **build_command_line_args(int argc, char *argv[], int *_argc) {
   file_info_t   *rcfile;
   char        **_argv = NULL;
   
-  _argv  = (char **) xine_xmalloc(sizeof(char **) * (argc + 1));
+  _argv  = (char **) calloc((argc + 1), sizeof(char *));
   (*_argc) = argc;
 
   _argv[0] = strdup(argv[0]);
   
-  rcfile           = (file_info_t *) xine_xmalloc(sizeof(file_info_t));
-  rcfile->filename = (char *) xine_xmalloc((strlen((xine_get_homedir())) + 
+  rcfile           = (file_info_t *) calloc(1, sizeof(file_info_t));
+  rcfile->filename = (char *) malloc((strlen((xine_get_homedir())) + 
 					    strlen(cfgdir) + strlen(xinerc)) + 3);
   sprintf(rcfile->filename, "%s/%s/%s", (xine_get_homedir()), cfgdir, xinerc);
   
@@ -439,13 +437,13 @@ static void show_banner(void) {
   
   show_version();
 
-  if(gGui->verbosity)
+  if(__xineui_global_verbosity)
     printf(_("Built with xine library %d.%d.%d (%s)\n"),
 	    XINE_MAJOR_VERSION, XINE_MINOR_VERSION, XINE_SUB_VERSION, XINE_VERSION);
   
   xine_get_version (&major, &minor, &sub);
 
-  if(gGui->verbosity)
+  if(__xineui_global_verbosity)
     printf(_("Found xine library version: %d.%d.%d (%s).\n"), 
 	    major, minor, sub, xine_get_version_string());
 }
@@ -479,30 +477,29 @@ static void print_formatted(char *title, const char *const *plugins) {
   printf("%s.\n\n", buffer);
 }
 static void list_plugins(char *type) {
-  typedef struct {
-    const char *const *(*func)(xine_t *);
-    char                *name;
-    char                *type;
-  } _list_plugin_t;
   const char   *const  *plugins;
   xine_t               *xine;
   char                 *cfgdir     = CONFIGDIR;
   char                 *cfgfile    = CONFIGFILE;
   char                 *configfile = NULL;
   int                   i;
-  _list_plugin_t        list_functions[] = {
-    { xine_list_audio_output_plugins,  _("   -Audio output:\n"),    "audio_out"     },
-    { xine_list_video_output_plugins,  _("   -Video output:\n"),    "video_out"     },
-    { xine_list_demuxer_plugins,       _("   -Demuxer:\n"),         "demux"         },
-    { xine_list_input_plugins,         _("   -Input:\n"),           "input"         },
-    { xine_list_spu_plugins,           _("   -Subpicture:\n"),      "sub"           },
-    { xine_list_post_plugins,          _("   -Post processing:\n"), "post"          },
-    { xine_list_audio_decoder_plugins, _("   -Audio decoder:\n"),   "audio_decoder" },
-    { xine_list_video_decoder_plugins, _("   -Video decoder:\n"),   "video_decoder" },
-    { NULL,                            NULL,                        NULL            }
+  static const struct {
+    const char *const *(*func)(xine_t *);
+    char                name[24];
+    char                type[16];
+  } list_functions[] = {
+    { xine_list_audio_output_plugins,  N_("   -Audio output:\n"),    "audio_out"     },
+    { xine_list_video_output_plugins,  N_("   -Video output:\n"),    "video_out"     },
+    { xine_list_demuxer_plugins,       N_("   -Demuxer:\n"),         "demux"         },
+    { xine_list_input_plugins,         N_("   -Input:\n"),           "input"         },
+    { xine_list_spu_plugins,           N_("   -Subpicture:\n"),      "sub"           },
+    { xine_list_post_plugins,          N_("   -Post processing:\n"), "post"          },
+    { xine_list_audio_decoder_plugins, N_("   -Audio decoder:\n"),   "audio_decoder" },
+    { xine_list_video_decoder_plugins, N_("   -Video decoder:\n"),   "video_decoder" },
+    { NULL,                            "",                           ""              }
   };
     
-  configfile = (char *) xine_xmalloc(strlen(xine_get_homedir())
+  configfile = (char *) malloc(strlen(xine_get_homedir())
 				     + strlen(cfgdir) 
 				     + strlen(cfgfile)
 				     + 3);
@@ -519,12 +516,12 @@ static void list_plugins(char *type) {
   if(type && strlen(type)) {
     i = 0;
     
-    while(list_functions[i].name) {
+    while(list_functions[i].func) {
       
       if(!strncasecmp(type, list_functions[i].type, strlen(type))) {
 	if((plugins = list_functions[i].func(xine))) {
 	  printf(_("\n Available xine's plugins:\n"));
-	  print_formatted(list_functions[i].name, plugins);
+	  print_formatted(gettext(list_functions[i].name), plugins);
 	  goto __found;
 	}
       }
@@ -540,10 +537,10 @@ static void list_plugins(char *type) {
     
     i = 0;
     
-    while(list_functions[i].name) {
+    while(list_functions[i].func) {
 
       if((plugins = list_functions[i].func(xine)))
-	print_formatted(list_functions[i].name, plugins);
+	print_formatted(gettext(list_functions[i].name), plugins);
       
       i++;
     }
@@ -562,9 +559,9 @@ static void show_usage (void) {
   char         *cfgdir     = CONFIGDIR;
   char         *cfgfile    = CONFIGFILE;
   char         *configfile = NULL;
-  char        **backends, *backend;
+  const char  **backends, *backend;
   
-  configfile = (char *) xine_xmalloc(strlen(xine_get_homedir())
+  configfile = (char *) malloc(strlen(xine_get_homedir())
 				     + strlen(cfgdir) 
 				     + strlen(cfgfile)
 				     + 3);
@@ -754,11 +751,14 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
   
 #ifdef HAVE_XINERAMA
   if (XineramaQueryExtension(gGui->video_display, &dummy_event, &dummy_error)) {
+    void *info;
     int count = 1;
-    XineramaQueryScreens(gGui->video_display, &count);
+
+    info = XineramaQueryScreens(gGui->video_display, &count);
     if (count > 1)
       /* multihead -> assuming square pixels */
       gGui->pixel_aspect = 1.0;
+    if (info) XFree(info);
   }
 #endif
 
@@ -774,7 +774,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   driver_num = 
-    xine_config_register_enum(gGui->xine, "video.driver", 
+    xine_config_register_enum(__xineui_global_xine_instance, "video.driver", 
 			      0, video_driver_ids,
 			      _("video driver to use"),
 			      _("Choose video driver. "
@@ -790,7 +790,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
     
     if((!strcasecmp(video_driver_ids[driver_num], "none")) || 
        (!strcasecmp(video_driver_ids[driver_num], "null"))) {
-      video_port = xine_open_video_driver(gGui->xine,
+      video_port = xine_open_video_driver(__xineui_global_xine_instance,
 					  video_driver_ids[driver_num],
 					  XINE_VISUAL_TYPE_NONE,
 					  (void *) &vis);
@@ -799,7 +799,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
       
     }
     else if(!strcasecmp(video_driver_ids[driver_num], "fb")) {
-      video_port = xine_open_video_driver(gGui->xine,
+      video_port = xine_open_video_driver(__xineui_global_xine_instance,
 					  video_driver_ids[driver_num],
 					  XINE_VISUAL_TYPE_FB, NULL);
       if (video_port)
@@ -808,7 +808,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
     }
     else if(strcasecmp(video_driver_ids[driver_num], "auto")) {
       
-      video_port = xine_open_video_driver(gGui->xine, 
+      video_port = xine_open_video_driver(__xineui_global_xine_instance, 
 					  video_driver_ids[driver_num],
 					  XINE_VISUAL_TYPE_X11,
 					  (void *) &vis);
@@ -820,14 +820,14 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
      *       but doing it here should do no harm.
      */
     i = 0;
-    driver_ids = xine_list_video_output_plugins (gGui->xine);
+    driver_ids = xine_list_video_output_plugins (__xineui_global_xine_instance);
 
     while (driver_ids[i]) {
       
-      if(gGui->verbosity)
+      if(__xineui_global_verbosity)
 	printf (_("main: probing <%s> video output plugin\n"), driver_ids[i]);
       
-      video_port = xine_open_video_driver(gGui->xine, 
+      video_port = xine_open_video_driver(__xineui_global_xine_instance, 
 					  driver_ids[i],
 					  XINE_VISUAL_TYPE_X11, 
 					  (void *) &vis);
@@ -849,7 +849,7 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
     /* 'none' plugin is a special case, just change the visual type */
     if((!strcasecmp(video_driver_ids[driver_number], "none")) 
        || (!strcasecmp(video_driver_ids[driver_number], "null"))) {
-      video_port = xine_open_video_driver(gGui->xine,
+      video_port = xine_open_video_driver(__xineui_global_xine_instance,
 					  video_driver_ids[driver_number],
 					  XINE_VISUAL_TYPE_NONE,
 					  (void *) &vis);
@@ -857,13 +857,13 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
       /* do not save on config, otherwise user would never see images again... */
     }
     else if(!strcasecmp(video_driver_ids[driver_number], "fb")) {
-      video_port = xine_open_video_driver(gGui->xine,
+      video_port = xine_open_video_driver(__xineui_global_xine_instance,
 					  video_driver_ids[driver_number],
 					  XINE_VISUAL_TYPE_FB, NULL);
 
     }
     else {
-      video_port = xine_open_video_driver(gGui->xine,
+      video_port = xine_open_video_driver(__xineui_global_xine_instance,
 					  video_driver_ids[driver_number],
 					  XINE_VISUAL_TYPE_X11, 
 					  (void *) &vis);
@@ -890,7 +890,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   driver_num = 
-    xine_config_register_enum(gGui->xine, "video.driver", 
+    xine_config_register_enum(__xineui_global_xine_instance, "video.driver", 
 			      0, video_driver_ids,
 			      _("video driver to use"),
 			      _("Choose video driver. "
@@ -901,7 +901,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
   
 
   driver_num = 
-    xine_config_register_enum(gGui->xine, "audio.driver", 
+    xine_config_register_enum(__xineui_global_xine_instance, "audio.driver", 
 			      0, audio_driver_ids,
 			      _("audio driver to use"),
 			      _("Choose audio driver. "
@@ -918,12 +918,12 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
       
       /* don't want to load an audio driver ? */
       if (!strncasecmp(audio_driver_ids[driver_num], "NULL", 4)) {
-	if(gGui->verbosity)
+	if(__xineui_global_verbosity)
 	  printf(_("main: not using any audio driver (as requested).\n"));
         return NULL;
       }
       
-      audio_port = xine_open_audio_driver(gGui->xine, 
+      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, 
 					  audio_driver_ids[driver_num],
 					  NULL);
       if (audio_port)
@@ -934,22 +934,22 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
      *       but doing it here should do no harm.
      */
     i = 0;
-    driver_ids = xine_list_audio_output_plugins (gGui->xine);
+    driver_ids = xine_list_audio_output_plugins (__xineui_global_xine_instance);
 
     while (driver_ids[i]) {
       
       if(strcmp(driver_ids[i], "none")) {
-	if(gGui->verbosity)
+	if(__xineui_global_verbosity)
 	  printf(_("main: probing <%s> audio output plugin\n"), driver_ids[i]);
       }
       else {
-	if(gGui->verbosity)
+	if(__xineui_global_verbosity)
 	  printf(_("main: skipping <%s> audio output plugin from auto-probe\n"), driver_ids[i]);
 	i++;
         continue;
       }
       
-      audio_port = xine_open_audio_driver(gGui->xine, 
+      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, 
 					  driver_ids[i],
 					  NULL);
       if (audio_port) {
@@ -966,7 +966,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
     /* don't want to load an audio driver ? */
     if (!strncasecmp (audio_driver_ids[driver_number], "NULL", 4)) {
 
-      if(gGui->verbosity)
+      if(__xineui_global_verbosity)
 	printf(_("main: not using any audio driver (as requested).\n"));
 
       /* calling -A null is useful to developers, but we should not save it at
@@ -977,7 +977,7 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
     }
     else {
     
-      audio_port = xine_open_audio_driver(gGui->xine, audio_driver_ids[driver_number], NULL);
+      audio_port = xine_open_audio_driver(__xineui_global_xine_instance, audio_driver_ids[driver_number], NULL);
 
       if (!audio_port) {
         printf (_("main: audio driver <%s> failed\n"), audio_driver_ids[driver_number]);
@@ -1011,6 +1011,9 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     return;
   }
   
+  if(gGui->stdctl_enable)
+    stdctl_event(event);
+
   switch(event->type) { 
     
   /* frontend can e.g. move on to next playlist entry */
@@ -1041,13 +1044,11 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       
       if(strcmp(gGui->mmk.ident, uevent->str)) {
 	
-	if(gGui->mmk.ident)
-	  free(gGui->mmk.ident);
+	free(gGui->mmk.ident);
 	if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
 	   gGui->playlist.mmk[gGui->playlist.cur]) {
 	  
-	  if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
-	    free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
+	  free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
 	  
 	  gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(uevent->str);
 	}
@@ -1122,60 +1123,60 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 
         /* (host name) */
       case XINE_MSG_UNKNOWN_HOST:
-	snprintf(buffer, sizeof(buffer), "%s", _("The host you're trying to connect is unknown.\n"
-						 "Check the validity of the specified hostname."));
+	strlcpy(buffer, _("The host you're trying to connect is unknown.\n"
+			  "Check the validity of the specified hostname."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 	
 	/* (device name) */
       case XINE_MSG_UNKNOWN_DEVICE:
-	snprintf(buffer, sizeof(buffer), "%s", _("The device name you specified seems invalid."));
+	strlcpy(buffer, _("The device name you specified seems invalid."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_NETWORK_UNREACHABLE:
-	snprintf(buffer, sizeof(buffer), "%s", _("The network looks unreachable.\nCheck your network "
-						 "setup and/or the server name."));
+	strlcpy(buffer, _("The network looks unreachable.\nCheck your network "
+			  "setup and/or the server name."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (host name) */
       case XINE_MSG_CONNECTION_REFUSED:
-	snprintf(buffer, sizeof(buffer), "%s", _("The connection was refused.\nCheck the host name."));
+	strlcpy(buffer, _("The connection was refused.\nCheck the host name."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (file name or mrl) */
       case XINE_MSG_FILE_NOT_FOUND:
-	snprintf(buffer, sizeof(buffer), "%s", _("The specified file or MRL could not be found. Please check it twice."));
+	strlcpy(buffer, _("The specified file or MRL could not be found. Please check it twice."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 
 	/* (device/file/mrl) */
       case XINE_MSG_READ_ERROR:
-	snprintf(buffer, sizeof(buffer), "%s", _("The source can't be read.\nMaybe you don't have enough "
-						 "rights for this, or source doesn't contain data "
-						 "(e.g: not disc in drive)."));
+	strlcpy(buffer, _("The source can't be read.\nMaybe you don't have enough "
+			  "rights for this, or source doesn't contain data "
+			  "(e.g: not disc in drive)."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s)", (char *) data + data->parameters);
 	break;
 	
 	/* (library/decoder) */
       case XINE_MSG_LIBRARY_LOAD_ERROR:
-	snprintf(buffer, sizeof(buffer), "%s", _("A problem occurred while loading a library or a decoder"));
+	strlcpy(buffer, _("A problem occurred while loading a library or a decoder"), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), ": %s", (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_ENCRYPTED_SOURCE:
-	snprintf(buffer, sizeof(buffer), "%s", _("The source seems encrypted, and can't be read."));
+	strlcpy(buffer, _("The source seems encrypted, and can't be read."), sizeof(buffer));
 	if(!strncasecmp(gGui->mmk.mrl, "dvd:/", 5)) {
 	  strlcat(buffer, _("\nYour DVD is probably crypted. "
 			   "According to your country laws, you can or can't "
@@ -1192,23 +1193,23 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	if(data->explanation)
 	  snprintf(buffer, sizeof(buffer), "%s %s", (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  snprintf(buffer, sizeof(buffer), "%s", _("No information available."));
+	  strlcpy(buffer, _("No information available."), sizeof(buffer));
 	break;
 
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
 	gui_stop(NULL, NULL);
-	snprintf(buffer, sizeof(buffer), "%s", _("The audio device is unavailable. "
-						 "Please verify if another program already uses it."));
+	strlcpy(buffer, _("The audio device is unavailable. "
+			  "Please verify if another program already uses it."), sizeof(buffer));
 	break;
 
       default:
-	snprintf(buffer, sizeof(buffer), "%s", _("*sight*, unkown error."));
+	strlcpy(buffer, _("*sight*, unkown error."), sizeof(buffer));
 	if(data->explanation)
 	  snprintf(buffer+strlen(buffer), sizeof(buffer)-strlen(buffer), " (%s %s)", (char *) data + data->explanation, (char *) data + data->parameters);
 	break;
       }
       
-      if(gGui->verbosity >= XINE_VERBOSITY_DEBUG) {
+      if(__xineui_global_verbosity >= XINE_VERBOSITY_DEBUG) {
 	strlcat(buffer, "\n\n[", sizeof(buffer));
 	
 	if(data->explanation)
@@ -1278,7 +1279,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
     if((event->stream == gGui->stream) && gGui->playlist.num) {
       xine_mrl_reference_data_t *ref = (xine_mrl_reference_data_t *) event->data;
 
-      if(gGui->verbosity)
+      if(__xineui_global_verbosity)
 	printf("XINE_EVENT_MRL_REFERENCE got mrl [%s] (alternative=%d)\n",
                ref->mrl, ref->alternative);
 
@@ -1351,7 +1352,12 @@ int main(int argc, char *argv[]) {
   char                  **session_argv     = NULL;
   int                     session_argv_num = 0;
   int                     retval           = 0;
+  pthread_mutexattr_t     mutexattr;
   
+  /* Set stdout always line buffered to get every     */
+  /* message line immediately, needed esp. for stdctl */
+  setlinebuf(stdout);
+
 #ifdef ENABLE_NLS
   if((xitk_set_locale()) != NULL) {
     setlocale(LC_ALL, "");
@@ -1377,7 +1383,7 @@ int main(int argc, char *argv[]) {
   if (sigprocmask (SIG_BLOCK,  &vo_mask, NULL))
     fprintf (stderr, "sigprocmask() failed.\n");
 
-  gGui = (gGui_t *) xine_xmalloc(sizeof(gGui_t));
+  gGui = (gGui_t *) calloc(1, sizeof(gGui_t));
   
   gGui->stream                 = NULL;
   gGui->debug_level            = 0;
@@ -1399,7 +1405,7 @@ int main(int argc, char *argv[]) {
   gGui->playlist.loop          = PLAYLIST_LOOP_NO_LOOP;
   gGui->playlist.control       = 0;
   gGui->skin_server_url        = NULL;
-  gGui->verbosity              = 0;
+  __xineui_global_verbosity              = 0;
   gGui->broadcast_port         = 0;
   gGui->display_logo           = 1;
   gGui->post_video_elements    = NULL;
@@ -1410,7 +1416,7 @@ int main(int argc, char *argv[]) {
   gGui->post_audio_enable      = 1;
   gGui->splash                 = 1;
 #ifdef HAVE_LIRC
-  gGui->lirc_enable            = 1;
+  __xineui_global_lirc_enable            = 1;
 #endif
   gGui->deinterlace_enable     = 0;
   gGui->report                 = stdout;
@@ -1419,6 +1425,7 @@ int main(int argc, char *argv[]) {
   gGui->no_mouse               = 0;
   gGui->wid                    = 0;
   gGui->nongui_error_msg       = NULL;
+  gGui->stdout                 = stdout;
   
   window_attribute.x     = window_attribute.y      = -8192;
   window_attribute.width = window_attribute.height = -1;
@@ -1429,10 +1436,12 @@ int main(int argc, char *argv[]) {
   _rc_file_check_args(_argc, _argv);
 #endif
 
+  pthread_mutex_init(&gGui->mmk_mutex, NULL);
+
   visual_anim_init();
 
   /*
-   * Parse X Ressource database
+   * Parse X Resource database
    */
   xrm_parse();
 
@@ -1444,10 +1453,9 @@ int main(int argc, char *argv[]) {
 			 long_options, &option_index)) != EOF) {
     switch(c) {
 
-
 #ifdef HAVE_LIRC
     case 'L': /* Disable LIRC support */
-      gGui->lirc_enable = 0;
+      __xineui_global_lirc_enable = 0;
       break;
 #endif
 
@@ -1560,7 +1568,7 @@ int main(int argc, char *argv[]) {
 	else if(!strcasecmp(p, "dvb"))
 	  aspect_ratio = XINE_VO_ASPECT_DVB;
 	else {
-	  printf(_("Bad aspect ratio mode '%s', see xine --help\n"), optarg);
+	  fprintf(stderr, _("Bad aspect ratio mode '%s', see xine --help\n"), optarg);
 	  exit(1);
 	}
       }
@@ -1624,7 +1632,7 @@ int main(int argc, char *argv[]) {
 	if((port >= 1) && (port <= 65535))
 	  gGui->network_port = port;
 	else
-	  printf(_("Bad port number: %d\n"), port);
+	  fprintf(stderr, _("Bad port number: %d\n"), port);
 
       }
       break;
@@ -1636,7 +1644,7 @@ int main(int argc, char *argv[]) {
     case 'G': /* Set geometry */
       if(optarg != NULL) {
 	if(!parse_geometry(&window_attribute, optarg)) {
-	  printf(_("Bad geometry '%s', see xine --help\n"), optarg);
+	  fprintf(stderr, _("Bad geometry '%s', see xine --help\n"), optarg);
 	  exit(1);
 	}
       }
@@ -1676,7 +1684,7 @@ int main(int argc, char *argv[]) {
 	else if(!strcasecmp(p, "shuffle"))
 	  gGui->playlist.loop = PLAYLIST_LOOP_SHUFFLE;
 	else {
-	  printf(_("Bad loop mode '%s', see xine --help\n"), optarg);
+	  fprintf(stderr, _("Bad loop mode '%s', see xine --help\n"), optarg);
 	  exit(1);
 	}
       }
@@ -1697,9 +1705,9 @@ int main(int argc, char *argv[]) {
 
     case OPTION_VERBOSE:
       if(optarg != NULL)
-	gGui->verbosity = strtol(optarg, &optarg, 10);
+	__xineui_global_verbosity = strtol(optarg, &optarg, 10);
       else
-	gGui->verbosity = 1;
+	__xineui_global_verbosity = 1;
       break;
 
 #ifdef XINE_PARAM_BROADCASTER_PORT    
@@ -1740,7 +1748,7 @@ int main(int argc, char *argv[]) {
 	
 	memset(&buffer, 0, sizeof(buffer));
 	xitk_subst_special_chars(cfg, &buffer[0]);
-	gGui->configfile = strdup(buffer);
+	__xineui_global_config_file = strdup(buffer);
       }
       break;
 
@@ -1821,7 +1829,7 @@ int main(int argc, char *argv[]) {
 	  }
 
 	  gGui->report = f;
-	  gGui->verbosity = 0xff;
+	  __xineui_global_verbosity = 0xff;
 	}
 	
 	if(optarg) {
@@ -1829,7 +1837,7 @@ int main(int argc, char *argv[]) {
 
 	  session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
 	  
-	  session_argv[session_argv_num] = (char *) xine_xmalloc(strlen(p) + 5);
+	  session_argv[session_argv_num] = (char *) malloc(strlen(p) + 5);
 	  sprintf(session_argv[session_argv_num], "mrl=%s", p);
 	  session_argv[++session_argv_num]   = NULL;
 
@@ -1877,7 +1885,7 @@ int main(int argc, char *argv[]) {
 	}
       }
       else
-	printf(_("You should specify at least one MRL to enqueue!\n"));
+	fprintf(stderr, _("You should specify at least one MRL to enqueue!\n"));
 
     }
 
@@ -1899,14 +1907,23 @@ int main(int argc, char *argv[]) {
   show_banner();
   
 #ifndef DEBUG
-  if(!gGui->verbosity) {
-    int error_fd;
+  /* Make non-verbose stdout really quiet but keep a copy for special use, e.g. stdctl feedback */
+  if(!__xineui_global_verbosity) {
+    int   guiout_fd, stdout_fd;
+    FILE *guiout_fp;
     
-    if ((error_fd = open("/dev/null", O_WRONLY)) < 0)
-      printf("cannot open /dev/null");
+    if((guiout_fd = dup(STDOUT_FILENO)) < 0)
+      fprintf(stderr, "cannot dup STDOUT_FILENO: %s.\n", strerror(errno));
+    else if((guiout_fp = fdopen(guiout_fd, "w")) == NULL)
+      fprintf(stderr, "cannot fdopen guiout_fd: %s.\n", strerror(errno));
+    else if((stdout_fd = open("/dev/null", O_WRONLY)) < 0)
+      fprintf(stderr, "cannot open /dev/null: %s.\n", strerror(errno));
+    else if(dup2(stdout_fd, STDOUT_FILENO) < 0)
+      fprintf(stderr, "cannot dup2 stdout_fd: %s.\n", strerror(errno));
     else {
-      if (dup2(error_fd, STDOUT_FILENO) < 0)
-	printf("cannot dup2 stdout");
+      gGui->stdout = guiout_fp;
+      setlinebuf(gGui->stdout);
+      close(stdout_fd); /* stdout_fd was intermediate, not needed any longer */
     }
   }
 #endif
@@ -1914,19 +1931,19 @@ int main(int argc, char *argv[]) {
   /*
    * Initialize config
    */
-  if(gGui->configfile == NULL) {
+  if(__xineui_global_config_file == NULL) {
     struct stat st;
     
-    gGui->configfile = (char *) xine_xmalloc(strlen(xine_get_homedir())
+    __xineui_global_config_file = (char *) malloc(strlen(xine_get_homedir())
 					     + strlen(cfgdir) 
 					     + strlen(cfgfile)
 					     + 3);
-    sprintf (gGui->configfile, "%s/%s", xine_get_homedir(), cfgdir);
-    mkdir (gGui->configfile, 0755);
-    sprintf (gGui->configfile + strlen(gGui->configfile), "/%s", cfgfile);
+    sprintf (__xineui_global_config_file, "%s/%s", xine_get_homedir(), cfgdir);
+    mkdir (__xineui_global_config_file, 0755);
+    sprintf (__xineui_global_config_file + strlen(__xineui_global_config_file), "/%s", cfgfile);
     
     /* Popup setup window if there is no config file */
-    if(stat(gGui->configfile, &st) < 0) {
+    if(stat(__xineui_global_config_file, &st) < 0) {
       if(aos < MAX_ACTIONS_ON_START)
 	gGui->actions_on_start[aos++] = ACTID_SETUP;
     }
@@ -1940,7 +1957,7 @@ int main(int argc, char *argv[]) {
     char *cfgdir = CONFIGDIR;
     char *keymap = "keymap";
 
-    gGui->keymap_file = (char *) xine_xmalloc(strlen(xine_get_homedir())
+    gGui->keymap_file = (char *) malloc(strlen(xine_get_homedir())
 					      + strlen(cfgdir) 
 					      + strlen(cfgfile)
 					      + strlen(keymap) + 3);
@@ -1949,15 +1966,15 @@ int main(int argc, char *argv[]) {
   
   pthread_mutex_init(&gGui->xe_mutex, NULL);
 
-  gGui->xine = xine_new();
-  xine_config_load(gGui->xine, gGui->configfile);
-  xine_engine_set_param(gGui->xine, XINE_ENGINE_PARAM_VERBOSITY, gGui->verbosity);
+  __xineui_global_xine_instance = xine_new();
+  xine_config_load(__xineui_global_xine_instance, __xineui_global_config_file);
+  xine_engine_set_param(__xineui_global_xine_instance, XINE_ENGINE_PARAM_VERBOSITY, __xineui_global_verbosity);
   
   /* 
    * Playlist auto reload 
    */
   old_playlist_cfg = 
-    xine_config_register_bool(gGui->xine, "gui.playlist_auto_reload", 
+    xine_config_register_bool(__xineui_global_xine_instance, "gui.playlist_auto_reload", 
 			      0,
 			      _("Automatically reload old playlist"),
 			      _("If it's enabled and if you don't specify any MRL in command "
@@ -1974,7 +1991,7 @@ int main(int argc, char *argv[]) {
   }
 
   gGui->subtitle_autoload = 
-    xine_config_register_bool(gGui->xine, "gui.subtitle_autoload", 1,
+    xine_config_register_bool(__xineui_global_xine_instance, "gui.subtitle_autoload", 1,
 			      _("Subtitle autoloading"),
 			      _("Automatically load subtitles if they exist."), 
 			      CONFIG_LEVEL_BEG,
@@ -1983,7 +2000,7 @@ int main(int argc, char *argv[]) {
 
 
   enable_deinterlace +=
-    xine_config_register_bool(gGui->xine, "gui.deinterlace_by_default", 0,
+    xine_config_register_bool(__xineui_global_xine_instance, "gui.deinterlace_by_default", 0,
 			      _("Enable deinterlacing by default"),
 			      _("Deinterlace plugin will be enabled on "
 			        "startup. Progressive streams are automaticaly "
@@ -2004,6 +2021,11 @@ int main(int argc, char *argv[]) {
 
   pthread_mutex_init(&gGui->download_mutex, NULL);
 
+  pthread_mutexattr_init(&mutexattr);
+  pthread_mutexattr_settype(&mutexattr, PTHREAD_MUTEX_RECURSIVE);
+  pthread_mutex_init(&gGui->logo_mutex, &mutexattr);
+  pthread_mutexattr_destroy(&mutexattr);
+
   /* Automatically start playback if new_mode is enabled and playlist is filled */
   if((gGui->smart_mode && 
       (gGui->playlist.num || actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) &&
@@ -2015,14 +2037,14 @@ int main(int argc, char *argv[]) {
   /*
    * xine init
    */
-  xine_init(gGui->xine);
+  xine_init(__xineui_global_xine_instance);
 
   /* Get old working path from input plugin */
   {
     xine_cfg_entry_t  cfg_entry;
     
-    if(xine_config_lookup_entry(gGui->xine, "media.files.origin_path", &cfg_entry))
-      snprintf(gGui->curdir, sizeof(gGui->curdir), "%s", cfg_entry.str_value);
+    if(xine_config_lookup_entry(__xineui_global_xine_instance, "media.files.origin_path", &cfg_entry))
+      strlcpy(gGui->curdir, cfg_entry.str_value, sizeof(gGui->curdir));
     else
       getcwd(&(gGui->curdir[0]), XITK_PATH_MAX);
   }
@@ -2033,12 +2055,12 @@ int main(int argc, char *argv[]) {
   /* Video out plugin */
   driver_num = -1;
   {
-    const char *const *vids = xine_list_video_output_plugins(gGui->xine);
+    const char *const *vids = xine_list_video_output_plugins(__xineui_global_xine_instance);
     int                i = 0;
     
     while(vids[i++]);
     
-    video_driver_ids = (char **) xine_xmalloc(sizeof(char *) * (i + 1));
+    video_driver_ids = (char **) calloc((i + 1), sizeof(char *));
     i = 0;
     video_driver_ids[i] = strdup("auto");
     while(vids[i]) {
@@ -2062,12 +2084,12 @@ int main(int argc, char *argv[]) {
   {
     xine_cfg_entry_t  cfg_vo_entry;
     
-    if(xine_config_lookup_entry(gGui->xine, "video.driver", &cfg_vo_entry)) {
+    if(xine_config_lookup_entry(__xineui_global_xine_instance, "video.driver", &cfg_vo_entry)) {
 
       if(!strcasecmp(video_driver_ids[cfg_vo_entry.num_value], "dxr3")) {
 	xine_cfg_entry_t  cfg_entry;
 	
-	if(xine_config_lookup_entry(gGui->xine, "dxr3.output.mode", &cfg_entry)) {
+	if(xine_config_lookup_entry(__xineui_global_xine_instance, "dxr3.output.mode", &cfg_entry)) {
 	  if(((!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "letterboxed tv")) ||
 	      (!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "widescreen tv"))) && 
 	     (!(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)))) {
@@ -2088,12 +2110,12 @@ int main(int argc, char *argv[]) {
   /* Audio out plugin */
   driver_num = -1;
   {
-    const char *const *aids = xine_list_audio_output_plugins(gGui->xine);
+    const char *const *aids = xine_list_audio_output_plugins(__xineui_global_xine_instance);
     int                i = 0;
     
     while(aids[i++]);
     
-    audio_driver_ids = (char **) xine_xmalloc(sizeof(char *) * (i + 2));
+    audio_driver_ids = (char **) calloc((i + 2), sizeof(char *));
     i = 0;
     audio_driver_ids[i] = strdup("auto");
     audio_driver_ids[i + 1] = strdup("null");
@@ -2119,7 +2141,7 @@ int main(int argc, char *argv[]) {
   post_deinterlace_init(pdeinterlace);
   post_init();
 
-  gGui->stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
+  gGui->stream = xine_stream_new(__xineui_global_xine_instance, gGui->ao_port, gGui->vo_port);
 #ifdef XINE_PARAM_EARLY_FINISHED_EVENT
   if( xine_check_version(1,1,1) )
       xine_set_param(gGui->stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1);
@@ -2129,8 +2151,8 @@ int main(int argc, char *argv[]) {
   if(gGui->ao_port && (gGui->mixer.method == SOUND_CARD_MIXER))
     gGui->mixer.original_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
 
-  gGui->vo_none = xine_open_video_driver(gGui->xine, "none", XINE_VISUAL_TYPE_NONE, NULL);
-  gGui->ao_none = xine_open_audio_driver(gGui->xine, "none", NULL);
+  gGui->vo_none = xine_open_video_driver(__xineui_global_xine_instance, "none", XINE_VISUAL_TYPE_NONE, NULL);
+  gGui->ao_none = xine_open_audio_driver(__xineui_global_xine_instance, "none", NULL);
 
   osd_init();
 
@@ -2139,7 +2161,7 @@ int main(int argc, char *argv[]) {
    */
   gGui->logo_mode = 0;
   gGui->logo_has_changed = 0;
-  gGui->logo_mrl = xine_config_register_string (gGui->xine, "gui.logo_mrl", XINE_LOGO_MRL,
+  gGui->logo_mrl = xine_config_register_string (__xineui_global_xine_instance, "gui.logo_mrl", XINE_LOGO_MRL,
 						_("Logo MRL"),
 						CONFIG_NO_HELP, 
 						CONFIG_LEVEL_EXP,
@@ -2163,7 +2185,7 @@ int main(int argc, char *argv[]) {
 #endif
 
   /* Visual animation stream init */
-  gGui->visual_anim.stream = xine_stream_new(gGui->xine, NULL, gGui->vo_port);
+  gGui->visual_anim.stream = xine_stream_new(__xineui_global_xine_instance, NULL, gGui->vo_port);
   gGui->visual_anim.event_queue = xine_event_new_queue(gGui->visual_anim.stream);
   gGui->visual_anim.current = 0;
   xine_event_create_listener_thread(gGui->visual_anim.event_queue, event_listener, NULL);
@@ -2172,7 +2194,7 @@ int main(int argc, char *argv[]) {
   xine_set_param(gGui->visual_anim.stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
   /* subtitle stream */
-  gGui->spu_stream = xine_stream_new(gGui->xine, NULL, gGui->vo_port);
+  gGui->spu_stream = xine_stream_new(__xineui_global_xine_instance, NULL, gGui->vo_port);
   xine_set_param(gGui->spu_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
   /* init the video window */
@@ -2183,8 +2205,7 @@ int main(int argc, char *argv[]) {
   /*
    * hand control over to gui
    */
-  if(aos < MAX_ACTIONS_ON_START)
-    gGui->actions_on_start[aos] = ACTID_NOKEY;
+  gGui->actions_on_start[aos] = ACTID_NOKEY;
   
   /* Initialize posts, if required */
   if(pplugins_num) {
@@ -2202,8 +2223,8 @@ int main(int argc, char *argv[]) {
 
   gui_run(session_argv);
 
-  if(pplugins)
-    free(pplugins);
+  visual_anim_done();
+  free(pplugins);
   
   if(session_argv_num) {
     int i = 0;
@@ -2214,11 +2235,15 @@ int main(int argc, char *argv[]) {
     free(session_argv);
   }
 
+  pthread_mutex_destroy(&gGui->mmk_mutex);
   pthread_mutex_destroy(&gGui->xe_mutex);
   pthread_mutex_destroy(&gGui->download_mutex);
+  pthread_mutex_destroy(&gGui->logo_mutex);
 
   if(gGui->report != stdout)
     fclose(gGui->report);
+  if(gGui->stdout != stdout)
+    fclose(gGui->stdout);
 
   free_command_line_args(&_argv, _argc);
 
