@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: mrl_browser.c,v 1.79 2006/07/15 20:26:20 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * MRL Browser
  *
@@ -33,7 +31,6 @@
 
 #include "common.h"
 
-extern gGui_t            *gGui;
 static xitk_widget_t     *mrlb = NULL;
 
 /*
@@ -179,16 +176,16 @@ static void mrl_browser_kill(xitk_widget_t *w, void *data) {
 static xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
   xitk_mrlbrowser_filter_t **filters = NULL;
   int                        num_endings = 0;
-  char                      *mrl_exts, *p, *pp;
+  char                      *mrl_exts, *pmrl_exts, *p, *pp;
   
   filters                      = (xitk_mrlbrowser_filter_t **) 
-    xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t *) * (num_endings + 2));
+    calloc((num_endings + 2), sizeof(xitk_mrlbrowser_filter_t *));
   filters[num_endings]         = (xitk_mrlbrowser_filter_t *)
     xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
   filters[num_endings]->name   = strdup("All");
   filters[num_endings]->ending = strdup("*");
 
-  mrl_exts = xine_get_file_extensions(gGui->xine);
+  mrl_exts = xine_get_file_extensions(__xineui_global_xine_instance);
   if(mrl_exts) {
     char  patterns[2048];
     char *e;
@@ -208,7 +205,8 @@ static xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
     filters[num_endings]->name   = strdup(_("All extensions"));
     filters[num_endings]->ending = pp;
 
-    while((e = xine_strsep(&mrl_exts, " ")) != NULL) {
+    pmrl_exts = mrl_exts;
+    while((e = xine_strsep(&pmrl_exts, " ")) != NULL) {
       
       snprintf(patterns, sizeof(patterns), "*.%s", e);
 
@@ -242,7 +240,7 @@ static xitk_mrlbrowser_filter_t **mrl_browser_get_valid_mrl_ending(void) {
 void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t play_cb,
 		 select_cb_t sel_cb, xitk_dnd_callback_t dnd_cb) {
   xitk_mrlbrowser_widget_t     mb;
-  const char *const           *ip_availables = xine_get_browsable_input_plugin_ids(gGui->xine);
+  const char *const           *ip_availables = xine_get_browsable_input_plugin_ids(__xineui_global_xine_instance);
   xitk_mrlbrowser_filter_t   **mrl_filters = mrl_browser_get_valid_mrl_ending();
 
   if(mrlb != NULL) {
@@ -258,14 +256,14 @@ void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t play_cb,
   mb.icon                           = &gGui->icon;
   mb.set_wm_window_normal           = !video_window_is_visible();
 
-  mb.x                              = xine_config_register_num (gGui->xine, "gui.mrl_browser_x", 
+  mb.x                              = xine_config_register_num (__xineui_global_xine_instance, "gui.mrl_browser_x", 
 								200,
 								"gui mrl browser x coordinate",
 								CONFIG_NO_HELP,
 								CONFIG_LEVEL_DEB,
 								CONFIG_NO_CB,
 								CONFIG_NO_DATA);
-  mb.y                              = xine_config_register_num (gGui->xine, "gui.mrl_browser_y",
+  mb.y                              = xine_config_register_num (__xineui_global_xine_instance, "gui.mrl_browser_y",
 								100,
 								"gui mrl browser y coordinate",
 								CONFIG_NO_HELP,
@@ -302,7 +300,7 @@ void mrl_browser(xitk_mrl_callback_t add_cb, xitk_mrl_callback_t play_cb,
   mb.ip_name.label.skin_element_name       = "MrlPlugLabel";
   mb.ip_name.label.label_str               = _("Source:");
 
-  mb.xine                                  = (xine_t *)gGui->xine;
+  mb.xine                                  = (xine_t *)__xineui_global_xine_instance;
 
   /* The browser */
 
