@@ -1,6 +1,6 @@
 #warning IMPLEMENT POST SUPPORT
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -16,16 +16,15 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: network.c,v 1.60 2004/04/13 19:55:48 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * xine network related stuff
  *
  */
-/* required for getsubopt(); the __sun test avoids compilation problems on */
-/* solaris */
-#if ! defined (__sun__) && ! defined (__OpenBSD__) 
+/* required for getsubopt(); the __sun test avoids compilation problems on 
+    solaris. On FreeBSD defining this disable BSD functions to be visible
+    and remove INADDR_NONE */
+#if ! defined (__sun__) && ! defined (__OpenBSD__)  && ! defined(__FreeBSD__) && ! defined(__APPLE__)
 #define _XOPEN_SOURCE 500
 #endif
 /* required for strncasecmp() */
@@ -33,9 +32,7 @@
 /* required to enable POSIX variant of getpwuid_r on solaris */
 #define _POSIX_PTHREAD_SEMANTICS 1
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #ifdef HAVE_READLINE
 
@@ -56,9 +53,10 @@
 #include <unistd.h>
 #include <time.h>
 #include <sys/ioctl.h>
+#include <sys/select.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -87,6 +85,10 @@
 
 #ifdef NETWORK_CLIENT
 
+#ifndef INADDR_NONE
+#define INADDR_NONE ((unsigned long) -1)
+#endif
+
 #ifdef HAVE_GETOPT_LONG
 #  include <getopt.h>
 #else
@@ -94,8 +96,8 @@
 #endif
 
 /* options args */
-static const char *short_options = "?hH:P:ncv";
-static struct option long_options[] = {
+static const char short_options[] = "?hH:P:ncv";
+static const struct option long_options[] = {
   { "help"           , no_argument      , 0, 'h' },
   { "host"           , required_argument, 0, 'H' },
   { "port"           , required_argument, 0, 'P' },
@@ -187,7 +189,6 @@ static void do_halt(commands_t *, client_info_t *);
 static void do_snap(commands_t *, client_info_t *);
 
 static void handle_client_command(client_info_t *);
-static const char *get_homedir(void);
 
 #define MAX_NAME_LEN        16
 #define MAX_PASSWD_LEN      16
@@ -225,7 +226,10 @@ struct client_info_s {
   int                   finished;
 
   int                   socket;
-  struct sockaddr_in    sin;
+  union {
+    struct sockaddr_in  in;
+    struct sockaddr     sa;
+  } fsin;
   char                **actions;
 
   struct {
@@ -428,11 +432,10 @@ static struct {
   { NULL                 , 0                 }
 };
 
-extern gGui_t *gGui;
 
 #endif
 
-void sock_err(const char *error_msg, ...) {
+static void __attribute__ ((format (printf, 1, 2))) sock_err(const char *error_msg, ...) {
   va_list args;
   
   va_start(args, error_msg);
@@ -440,7 +443,7 @@ void sock_err(const char *error_msg, ...) {
   va_end(args);
 }
 
-int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
+static int sock_create(const char *service, const char *transport, struct sockaddr_in *sin) {
   struct servent    *iservice;
   struct protoent *itransport;
   int                sock;
@@ -483,57 +486,10 @@ int sock_create(const char *service, const char *transport, struct sockaddr_in *
   return sock;
 }
 
-int sock_client(const char *host, const char *service, const char *transport) {
-  struct sockaddr_in   sin;
-  struct hostent      *ihost;
-  int                  sock;
-  
-  sock = sock_create(service, transport, &sin);
-
-  if ((sin.sin_addr.s_addr = inet_addr(host)) == -1) {
-    ihost = gethostbyname(host);
-    
-    if(!ihost) {
-      sock_err("Unknown host: %s\n", host);
-      return -1;
-    }
-    memcpy(&sin.sin_addr, ihost->h_addr, ihost->h_length);
-  }
-  
-  if(connect(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-    int err = errno;
-
-    close(sock);
-    errno = err;
-    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
-    return -1;
-  }
-
-  return sock;
-}
-
-int sock_serv(const char *service, const char *transport, int queue_length) {
-  struct sockaddr_in  sin;
-  int                 sock;
-  int                 on = 1;
-
-  sock = sock_create(service, transport, &sin);
-  
-  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
-  
-  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0)
-    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
-  
-  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
-    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
-  
-  return sock;
-}
-
 /*
  * Check for socket validity.
  */
-int sock_check_opened(int socket) {
+static int sock_check_opened(int socket) {
   fd_set   readfds, writefds, exceptfds;
   int      retval;
   struct   timeval timeout;
@@ -559,46 +515,6 @@ int sock_check_opened(int socket) {
   return 0;
 }
 
-/*
- * Read from socket.
- */
-int sock_read(int socket, char *buf, int len) {
-  char    *pbuf;
-  int      r, rr;
-  void    *nl;
-  
-  if((socket < 0) || (buf == NULL))
-    return -1;
-
-  if(!sock_check_opened(socket))
-    return -1;
-  
-  if (--len < 1)
-    return(-1);
-  
-  pbuf = buf;
-  
-  do {
-    
-    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
-      return -1;
-
-    if((nl = memchr(pbuf, '\n', r)) != NULL)
-      r = ((char *) nl) - pbuf + 1;
-    
-    if((rr = read(socket, pbuf, r)) < 0)
-      return -1;
-    
-    pbuf += rr;
-    len -= rr;
-
-  } while((nl == NULL) && len);
-  
-  *pbuf = '\0';
-  
-  return (pbuf - buf);
-}
-
 /*
  * Write to socket.
  */
@@ -626,7 +542,7 @@ static int _sock_write(int socket, char *buf, int len) {
   return wlen;
 }
 
-static int __sock_write(int socket, int cr, char *msg, ...) {
+static int __attribute__ ((format (printf, 3, 4))) __sock_write(int socket, int cr, char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   
@@ -637,7 +553,7 @@ static int __sock_write(int socket, int cr, char *msg, ...) {
   /* Each line sent is '\n' terminated */
   if(cr) {
     if((buf[strlen(buf)] == '\0') && (buf[strlen(buf) - 1] != '\n'))
-      sprintf(buf, "%s%c", buf, '\n');
+      strlcat(buf, "\n", sizeof(buf));
   }
   
   return _sock_write(socket, buf, strlen(buf));
@@ -678,6 +594,37 @@ static char *_atoa(char *str) {
 }
 
 #ifdef NETWORK_CLIENT
+static int sock_client(const char *host, const char *service, const char *transport) {
+  union {
+    struct sockaddr_in in;
+    struct sockaddr sa;
+  } fsin;
+  struct hostent      *ihost;
+  int                  sock;
+  
+  sock = sock_create(service, transport, &fsin.in);
+
+  if ((fsin.in.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
+    ihost = gethostbyname(host);
+    
+    if(!ihost) {
+      sock_err("Unknown host: %s\n", host);
+      return -1;
+    }
+    memcpy(&fsin.in.sin_addr, ihost->h_addr_list[0], ihost->h_length);
+  }
+  
+  if(connect(sock, &fsin.sa, sizeof(fsin.in)) < 0) {
+    int err = errno;
+
+    close(sock);
+    errno = err;
+    sock_err("Unable to connect %s[%s]: %s\n", host, service, strerror(errno));
+    return -1;
+  }
+
+  return sock;
+}
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 91)
 #define write_to_console_unlocked(session, msg, args...)  __sock_write(session->console, 1, msg, ##args)
@@ -687,7 +634,7 @@ static char *_atoa(char *str) {
 #define write_to_console_unlocked_nocr(session, ...) __sock_write(session->console, 0, __VA_ARGS__)
 #endif
 
-static int write_to_console(session_t *session, const char *msg, ...) {
+static int __attribute__ ((format (printf, 2, 3))) write_to_console(session_t *session, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   int      err;
@@ -697,14 +644,14 @@ static int write_to_console(session_t *session, const char *msg, ...) {
   va_end(args);
   
   pthread_mutex_lock(&session->console_mutex);
-  err = write_to_console_unlocked(session, buf);
+  err = write_to_console_unlocked(session, "%s", buf);
   pthread_mutex_unlock(&session->console_mutex);
 
   return err;
 }
 
 #if 0
-static int write_to_console_nocr(session_t *session, const char *msg, ...) {
+static int __attribute__ ((format (printf, 2, 3)) write_to_console_nocr(session_t *session, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
   int      err;
@@ -722,22 +669,17 @@ static int write_to_console_nocr(session_t *session, const char *msg, ...) {
 #endif
 
 static void session_update_prompt(session_t *session) {
-  char buf[514];
-
   if(session == NULL)
     return;
 
-  if(session->socket >= 0) {
-    snprintf(buf, sizeof(buf), "%s:%s", session->host, session->port);
-  }
+  if(session->socket >= 0)
+    snprintf(session->prompt, sizeof(session->prompt), "[%s:%s]"PROGNAME" >", session->host, session->port);
   else
-    snprintf(buf, sizeof(buf), "%s", "******:****");
-
-  snprintf(session->prompt, sizeof(session->prompt), "[%s]%s >", buf, PROGNAME);
+    strlcpy(session->prompt, "[******:****]"PROGNAME" >", sizeof(session->prompt));
 }
 
 static void session_create_commands(session_t *session) {
-  int num_commands = (sizeof(client_commands) / sizeof(client_commands[0]));
+  static const size_t num_commands = (sizeof(client_commands) / sizeof(client_commands[0]));
   int i;
   
   if(session == NULL)
@@ -754,7 +696,7 @@ static void session_create_commands(session_t *session) {
     free(session_commands);
   }
   
-  session_commands = (session_commands_t **) malloc(sizeof(session_commands_t *) * num_commands);
+  session_commands = (session_commands_t **) calloc(num_commands, sizeof(session_commands_t *));
   for(i = 0; client_commands[i].command != NULL; i++) {
     session_commands[i] = (session_commands_t *) malloc(sizeof(session_commands_t));
     session_commands[i]->command   = strdup(client_commands[i].command);
@@ -776,16 +718,14 @@ static void session_create_commands(session_t *session) {
 static void client_noop(session_t *session, session_commands_t *command, const char *cmd) {
 }
 static void client_help(session_t *session, session_commands_t *command, const char *cmd) {
-  int i = 0, j;
-  int maxlen = 0;
+  int i = 0;
+  size_t maxlen = 0, j;
   int curpos = 0;
-  char buf[_BUFSIZ];
+  char buf[_BUFSIZ] = "Available commands are:\n       ";
   
   if((session == NULL) || (command == NULL))
     return;
   
-  memset(&buf, 0, sizeof(buf));
-
   while(session_commands[i]->command != NULL) {
     
     if(session_commands[i]->enable) {
@@ -799,21 +739,20 @@ static void client_help(session_t *session, session_commands_t *command, const c
   maxlen++; 
   i = 0;  
   
-  sprintf(buf, "%s", "Available commands are:\n       ");
   curpos += 7;
   
   while(session_commands[i]->command != NULL) {
     if(session_commands[i]->enable) {
       if((curpos + maxlen) >= 80) {
-	sprintf(buf, "%s\n       ", buf);
+	strlcat(buf, "\n       ", sizeof(buf));
 	curpos = 7;
       }
       
-      sprintf(buf, "%s%s", buf, session_commands[i]->command);
+      strlcat(buf, session_commands[i]->command, sizeof(buf));
       curpos += strlen(session_commands[i]->command);
       
       for(j = 0; j < (maxlen - strlen(session_commands[i]->command)); j++) {
-	sprintf(buf, "%s ", buf);
+	strlcat(buf, " ", sizeof(buf));
 	curpos++;
       }
     }
@@ -863,7 +802,7 @@ static void client_open(session_t *session, session_commands_t *command, const c
   }
   
   if(cmd) {
-    sprintf(buf, "%s", cmd);
+    strlcpy(buf, cmd, sizeof(buf));
     pbuf = buf;
     if((p = strchr(pbuf, ' ')) != NULL) {
       host = _atoa(p);
@@ -881,10 +820,10 @@ static void client_open(session_t *session, session_commands_t *command, const c
     
     if(host != NULL) {
       
-      snprintf(session->host, sizeof(session->host), "%s", host);
+      strlcpy(session->host, host, sizeof(session->host));
       
       if(port != NULL) 
-	snprintf(session->port, sizeof(session->port), "%s", port);
+	strlcpy(session->port, port, sizeof(session->port));
       
       session_create_commands(session);
       session->socket = sock_client(session->host, session->port, "tcp");
@@ -1021,7 +960,7 @@ static void *select_thread(void *data) {
 		if((special = strstr(obuffer, COMMANDS_PREFIX)) != NULL) {
 		  char  *p, *pp;
 		  int    special_length = strlen(special);
-		  int    i = (sizeof(client_commands) / sizeof(client_commands[0])) - 1;
+		  size_t i = (sizeof(client_commands) / sizeof(client_commands[0])) - 1;
 		  
 		  pp = special + 11;
 		  while((p = xine_strsep(&pp, "\t")) != NULL) {
@@ -1057,7 +996,7 @@ static void *select_thread(void *data) {
 		  write_to_console_unlocked_nocr(session, "\b \b");
 		  pos--;
 		}
-		write_to_console_unlocked(session, obuffer);
+		write_to_console_unlocked(session, "%s", obuffer);
 
 		rl_crlf();
 		rl_forced_update_display();
@@ -1093,8 +1032,7 @@ static void client_handle_command(session_t *session, const char *command) {
     return;
   
   /* Get only first arg of command */
-  memset(&cmd, 0, sizeof(cmd));
-  sprintf(cmd, "%s", command);
+  strlcpy(cmd, command, sizeof(cmd));
   if((p = strchr(cmd, ' ')) != NULL)
     *p = '\0';
   
@@ -1141,7 +1079,7 @@ static void client_handle_command(session_t *session, const char *command) {
 	  
 	  *pp = '\0';
 	  
-	  if((sock_write(session->socket, buf)) == -1) {
+	  if((sock_write(session->socket, "%s", buf)) == -1) {
 	    session->running = 0;
 	  }
 	}
@@ -1153,7 +1091,7 @@ static void client_handle_command(session_t *session, const char *command) {
   
   /* Perhaps a ';' separated commands, so send anyway to server */
   if(found == 0) {
-    sock_write(session->socket, (char *)command);
+    sock_write(session->socket, "%s", (char *)command);
   }
   
   if((!strncasecmp(cmd, "exit", strlen(cmd))) || (!strncasecmp(cmd, "halt", strlen(cmd)))) {
@@ -1167,13 +1105,13 @@ static void session_single_shot(session_t *session, int num_commands, char *comm
   int i;
   char buf[_BUFSIZ];
   
-  memset(&buf, 0, sizeof(buf));
+  buf[0] = 0;
 
   for(i = 0; i < num_commands; i++) {
-    if(strlen(buf))
-      sprintf(buf, "%s %s", buf, commands[i]);
+    if(buf[0])
+      sprintf(buf+strlen(buf), " %s", commands[i]);
     else
-      sprintf(buf, "%s", commands[i]);
+      strcpy(buf, commands[i]);
   }
 
   client_handle_command(session, buf);
@@ -1258,8 +1196,8 @@ int main(int argc, char **argv) {
 
   session.socket = -1;
   pthread_mutex_init(&session.console_mutex, NULL);
-  snprintf(session.host, sizeof(session.host), "%s", DEFAULT_SERVER);
-  snprintf(session.port, sizeof(session.port), "%s", DEFAULT_XINECTL_PORT);
+  strcpy(session.host, DEFAULT_SERVER);
+  strcpy(session.port, DEFAULT_XINECTL_PORT);
 
   opterr = 0;
   while((c = getopt_long(argc, argv, short_options, long_options, &option_index)) != EOF) {
@@ -1267,13 +1205,13 @@ int main(int argc, char **argv) {
       
     case 'H': /* Set host */
       if(optarg != NULL)
-	snprintf(session.host, sizeof(session.host), "%s", optarg);
+	strlcpy(session.host, optarg, sizeof(session.host));
       break;
       
     case 'P': /* Set port */
       if(optarg != NULL) {
 	port_set = 1;
-	snprintf(session.port, sizeof(session.port), "%s", optarg);
+	strlcpy(session.port, optarg, sizeof(session.port));
       }
       break;
 
@@ -1384,6 +1322,67 @@ int main(int argc, char **argv) {
 
 #else
 
+static int sock_serv(const char *service, const char *transport, int queue_length) {
+  union {
+    struct sockaddr_in in;
+    struct sockaddr sa;
+  } fsin;
+  int                 sock;
+  int                 on = 1;
+
+  sock = sock_create(service, transport, &fsin.in);
+  
+  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(int));
+  
+  if(bind(sock, &fsin.sa, sizeof(fsin.in)) < 0)
+    sock_err("Unable to link socket %s: %s\n", service, strerror(errno));
+  
+  if(strcmp(transport, "udp") && listen(sock, queue_length) < 0)
+    sock_err("Passive mode impossible on %s: %s\n", service, strerror(errno));
+  
+  return sock;
+}
+
+/*
+ * Read from socket.
+ */
+static int sock_read(int socket, char *buf, int len) {
+  char    *pbuf;
+  int      r, rr;
+  void    *nl;
+  
+  if((socket < 0) || (buf == NULL))
+    return -1;
+
+  if(!sock_check_opened(socket))
+    return -1;
+  
+  if (--len < 1)
+    return(-1);
+  
+  pbuf = buf;
+  
+  do {
+    
+    if((r = recv(socket, pbuf, len, MSG_PEEK)) <= 0)
+      return -1;
+
+    if((nl = memchr(pbuf, '\n', r)) != NULL)
+      r = ((char *) nl) - pbuf + 1;
+    
+    if((rr = read(socket, pbuf, r)) < 0)
+      return -1;
+    
+    pbuf += rr;
+    len -= rr;
+
+  } while((nl == NULL) && len);
+  
+  *pbuf = '\0';
+  
+  return (pbuf - buf);
+}
+
 /*
  * Password related
  */
@@ -1431,7 +1430,7 @@ static int _passwdfile_is_entry_valid(char *entry, char *name, char *passwd) {
   char buf[_BUFSIZ];
   char *n, *p;
   
-  sprintf(buf, "%s", entry);
+  strlcpy(buf, entry, sizeof(buf));
   n = buf;
   if((p = strrchr(buf, ':')) != NULL) {
     if(strlen(p) > 1) {
@@ -1445,8 +1444,8 @@ static int _passwdfile_is_entry_valid(char *entry, char *name, char *passwd) {
   }
   
   if((n != NULL) && (p != NULL)) {
-    snprintf(name, MAX_NAME_LEN, "%s", n);
-    snprintf(passwd, MAX_PASSWD_LEN, "%s", p);
+    strlcpy(name, n, MAX_NAME_LEN);
+    strlcpy(passwd, p, MAX_PASSWD_LEN);
     return 1;
   }
 
@@ -1644,7 +1643,7 @@ static const char *get_arg(client_info_t *client_info, int num) {
 /*
  * return 1 if *arg match with argument <pos>
  */
-int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
+static int is_arg_contain(client_info_t *client_info, int pos, const char *arg) {
   
   if(client_info && pos && ((arg != NULL) && (strlen(arg))) && (client_info->command.num_args >= pos)) {
     if(!strncmp(client_info->command.args[pos - 1], arg, strlen(client_info->command.args[pos - 1])))
@@ -1661,13 +1660,9 @@ static void set_command_line(client_info_t *client_info, char *line) {
 
   if(client_info && line && strlen(line)) {
 
-    if(client_info->command.line)
-      client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
-    else
-      client_info->command.line = (char *) xine_xmalloc(strlen(line) + 1);
+    client_info->command.line = (char *) realloc(client_info->command.line, strlen(line) + 1);
     
-    memset(client_info->command.line, 0, sizeof(client_info->command.line));
-    sprintf(client_info->command.line, "%s", line);
+    strcpy(client_info->command.line, line);
 
   }
 }
@@ -1708,25 +1703,24 @@ static void do_commands(commands_t *cmd, client_info_t *client_info) {
   int i = 0;
   char buf[_BUFSIZ];
 
-  memset(&buf, 0, sizeof(buf));
-  sprintf(buf, COMMANDS_PREFIX);
+  strcpy(buf, COMMANDS_PREFIX);
   
   while(commands[i].command != NULL) {
     if(commands[i].public) {
-      sprintf(buf, "%s\t%s", buf, commands[i].command);
+      snprintf(buf+strlen(buf), sizeof(buf)-strlen(buf), "\t%s", commands[i].command);
     }
     i++;
   }
-  sprintf(buf, "%s.\n", buf);
-  sock_write(client_info->socket, buf);
+  strlcat(buf, ".\n", sizeof(buf));
+  sock_write(client_info->socket, "%s", buf);
 }
 
 static void do_help(commands_t *cmd, client_info_t *client_info) {
-  char buf[_BUFSIZ];
+  char buf[_BUFSIZ] = "Available commands are:\n       ";
 
   if(!client_info->command.num_args) {
-    int i = 0, j;
-    int maxlen = 0;
+    int i = 0;
+    size_t maxlen = 0, j;
     int curpos = 0;
     
     while(commands[i].command != NULL) {
@@ -1742,29 +1736,28 @@ static void do_help(commands_t *cmd, client_info_t *client_info) {
     maxlen++; 
     i = 0;  
 
-    sprintf(buf, "Available commands are:\n       ");
     curpos += 7;
     
     while(commands[i].command != NULL) {
       if(commands[i].public) {
 	if((curpos + maxlen) >= 80) {
-	  sprintf(buf, "%s\n       ", buf);
+	  strlcat(buf, "\n       ", sizeof(buf));
 	  curpos = 7;
 	}
 	
-	sprintf(buf, "%s%s", buf, commands[i].command);
+	strlcat(buf, commands[i].command, sizeof(buf));
 	curpos += strlen(commands[i].command);
 	
 	for(j = 0; j < (maxlen - strlen(commands[i].command)); j++) {
-	  sprintf(buf, "%s ", buf);
+	  strlcat(buf, " ", sizeof(buf));
 	  curpos++;
 	}
       }
       i++;
     }
     
-    sprintf(buf, "%s\n", buf);
-    sock_write(client_info->socket, buf);
+    strlcat(buf, "\n", sizeof(buf));
+    sock_write(client_info->socket, "%s", buf);
   }
   else {
     int i;
@@ -1803,7 +1796,7 @@ static void do_auth(commands_t *cmd, client_info_t *client_info) {
       char *name;
       char *passwd;
       
-      sprintf(buf, "%s", get_arg(client_info, 1));
+      strlcpy(buf, get_arg(client_info, 1), sizeof(buf));
       name = buf;
       if((passwd = strrchr(buf, ':')) != NULL) {
 	if(strlen(passwd) > 1) {
@@ -1819,8 +1812,8 @@ static void do_auth(commands_t *cmd, client_info_t *client_info) {
 	memset(&client_info->name, 0, sizeof(client_info->name));
 	memset(&client_info->passwd, 0, sizeof(client_info->passwd));
 
-	snprintf(client_info->name, sizeof(client_info->name), "%s", name);
-	snprintf(client_info->passwd, sizeof(client_info->passwd), "%s", passwd);
+	strlcpy(client_info->name, name, sizeof(client_info->name));
+	strlcpy(client_info->passwd, passwd, sizeof(client_info->passwd));
 
 	check_client_auth(client_info);
       }
@@ -1847,9 +1840,9 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	xine_stop (gGui->stream);
 	gGui->playlist.cur--;
 	if ((gGui->playlist.cur >= 0) && (gGui->playlist.cur < gGui->playlist.num)) {
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	  (void) gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
-					gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset);
+					gGui->mmk.start, gGui->mmk.av_offset, gGui->mmk.spu_offset, 1);
   
 	} 
 	else {
@@ -1877,7 +1870,7 @@ static void do_mrl(commands_t *cmd, client_info_t *client_info) {
 	  xine_stop(gGui->stream);
 	  gGui->ignore_next = 0;
 	}
-	gui_set_current_mrl(gGui->playlist.mmk[gGui->playlist.num - 1]);
+	gui_set_current_mmk(gGui->playlist.mmk[gGui->playlist.num - 1]);
 	if(!(xine_open(gGui->stream, gGui->mmk.mrl) 
 	     && xine_play (gGui->stream, 0, gGui->mmk.start))) {
 	  handle_xine_error(client_info);
@@ -1923,12 +1916,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
       else if((first = is_arg_contain(client_info, 1, "first")) || is_arg_contain(client_info, 1, "last")) {
 
 	if(gGui->playlist.num) {
-	  int entry = gGui->playlist.cur;
-	  
-	  if(entry)
-	    entry = 0;
-	  else
-	    entry = gGui->playlist.num - 1;
+	  int entry = (first) ? 0 : gGui->playlist.num - 1;
 	  
 	  if(entry != gGui->playlist.cur) {
 	    
@@ -1939,7 +1927,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	    }
 
 	    gGui->playlist.cur = entry;
-	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	    gui_set_current_mmk(mediamark_get_current_mmk());
 	    gui_play(NULL, NULL);
 	  }
 	}
@@ -1950,7 +1938,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
       }
       else if(is_arg_contain(client_info, 1, "continue")) {
 	if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
-	  gGui->playlist.control &= !PLAYLIST_CONTROL_STOP;
+	  gGui->playlist.control &= ~PLAYLIST_CONTROL_STOP;
       }
 
     }
@@ -1963,7 +1951,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	
 	if((j >= 0) && (j <= gGui->playlist.num)) {
 	  gGui->playlist.cur = j;
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 
 	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP) {
 	    gGui->ignore_next = 1;
@@ -2002,8 +1990,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	    for(i = j; i < gGui->playlist.num; i++)
 	      gGui->playlist.mmk[i] = gGui->playlist.mmk[i + 1];
 	    
-	    gGui->playlist.mmk = (mediamark_t **) 
-	      realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
+	    gGui->playlist.mmk = (mediamark_t **) realloc(gGui->playlist.mmk, sizeof(mediamark_t *) * (gGui->playlist.num + 2));
 	    
 	    gGui->playlist.mmk[gGui->playlist.num + 1] = NULL;
 	    
@@ -2015,7 +2002,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  playlist_update_playlist();
 
 	if(gGui->playlist.num)
-	  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	  gui_set_current_mmk(mediamark_get_current_mmk());
 	else {
 	  
 	  if(is_playback_widgets_enabled())
@@ -2024,7 +2011,7 @@ static void do_playlist(commands_t *cmd, client_info_t *client_info) {
 	  if(xine_get_status(gGui->stream) != XINE_STATUS_STOP)
 	    gui_stop(NULL, NULL);
 	  
-	  gui_set_current_mrl(NULL);
+	  gui_set_current_mmk(NULL);
 	}
       }
     }
@@ -2092,22 +2079,22 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	char buf[64];
 	int  status;
 
-	sprintf(buf, "%s", "Current status: ");
+	strcpy(buf, "Current status: ");
 	status = xine_get_status(gGui->stream);
 	
 	if(status <= XINE_STATUS_QUIT)
-	  sprintf(buf, "%s%s", buf, status_struct[status].name);
+	  strlcat(buf, status_struct[status].name, sizeof(buf));
 	else
-	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
+	  strlcat(buf, "*UNKNOWN*", sizeof(buf));
 	
-	sprintf(buf, "%s%c", buf, '\n');
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s\n", buf);
       }
       else if(is_arg_contain(client_info, 1, "speed")) {
 	char buf[64];
-	int  speed = -1, i;
+	int  speed = -1;
+	size_t i;
 	
-	sprintf(buf, "%s", "Current speed: ");
+	strcpy(buf, "Current speed: ");
 	speed = xine_get_param(gGui->stream, XINE_PARAM_SPEED);
 
 	for(i = 0; speeds_struct[i].name != NULL; i++) {
@@ -2116,12 +2103,11 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 	}
 	
 	if(i < ((sizeof(speeds_struct) / sizeof(speeds_struct[0])) - 1))
-	  sprintf(buf, "%s%s", buf, speeds_struct[i].name);
+	  strlcat(buf, speeds_struct[i].name, sizeof(buf));
 	else
-	  sprintf(buf, "%s%s", buf, "*UNKNOWN*");
+	  strlcat(buf, "*UNKNOWN*", sizeof(buf));
 	
-	sprintf(buf, "%s%c", buf, '\n');
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s\n", buf);
       }
       else if(is_arg_contain(client_info, 1, "position")) {
 	char buf[64];
@@ -2132,8 +2118,8 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current position: %d\n",pos_time);
-	sock_write(client_info->socket, buf);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current position", pos_time);
+	sock_write(client_info->socket, "%s", buf);
       }
       else if(is_arg_contain(client_info, 1, "length")) {
 	char buf[64];
@@ -2144,37 +2130,34 @@ static void do_get(commands_t *cmd, client_info_t *client_info) {
 			    &pos_stream,
 			    &pos_time,
 			    &length_time);
-	sprintf(buf, "Current length: %d\n",length_time);
-	sock_write(client_info->socket, buf);
+	snprintf(buf, sizeof(buf), "%s: %d\n", "Current length", length_time);
+	sock_write(client_info->socket, "%s", buf);
       }
       else if(is_arg_contain(client_info, 1, "loop")) {
-	char buf[64];
-
-	sprintf(buf, "Current loop mode is: ");
+	char buf[64] = "Current loop mode is: ";
 
 	switch(gGui->playlist.loop) {
 	case PLAYLIST_LOOP_NO_LOOP:
-	  sprintf(buf, "%s'No Loop'", buf);
+	  strlcat(buf, "'No Loop'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_LOOP:
-	  sprintf(buf, "%s'Loop'", buf);
+	  strlcat(buf, "'Loop'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_REPEAT:
-	  sprintf(buf, "%s'Repeat'", buf);
+	  strlcat(buf, "'Repeat'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_SHUFFLE:
-	  sprintf(buf, "%s'Shuffle'", buf);
+	  strlcat(buf, "'Shuffle'", sizeof(buf));
 	  break;
 	case PLAYLIST_LOOP_SHUF_PLUS:	
-	  sprintf(buf, "%s'Shuffle forever'", buf);
+	  strlcat(buf, "'Shuffle forever'", sizeof(buf));
 	  break;
 	default:
-	  sprintf(buf, "%s'!!Unknown!!'", buf);
+	  strlcat(buf, "'!!Unknown!!'", sizeof(buf));
 	  break;
 	}
 
-	sprintf(buf, "%s.\n", buf);
-	sock_write(client_info->socket, buf);
+	sock_write(client_info->socket, "%s.\n", buf);
       }
     }
     else if(nargs >= 2) {
@@ -2379,6 +2362,10 @@ static void do_gui(commands_t *cmd, client_info_t *client_info) {
 	gui_vpp_show(NULL, NULL);
 	flushing++;
       }
+      else if(is_arg_contain(client_info, 1, "apost")) {
+	gui_app_show(NULL, NULL);
+	flushing++;
+      }
       else if(is_arg_contain(client_info, 1, "help")) {
 	gui_help_show(NULL, NULL);
 	flushing++;
@@ -2436,11 +2423,11 @@ static void do_event(commands_t *cmd, client_info_t *client_info) {
 	while(events_struct[i].name && strncmp(arg, events_struct[i].name, strlen(arg))) i++;
       }
 
-      if((i >= 0) && (i < ((sizeof(events_struct) / sizeof(events_struct[0])) - 1))) {
+      if((i >= 0) && ((size_t)i < ((sizeof(events_struct) / sizeof(events_struct[0])) - 1))) {
 	xine_event.type = events_struct[i].type;
       }
       else {
-	int   value;
+	size_t value;
 	int   input_numbers[] = {
 	  XINE_EVENT_INPUT_NUMBER_0, XINE_EVENT_INPUT_NUMBER_1, XINE_EVENT_INPUT_NUMBER_2,
 	  XINE_EVENT_INPUT_NUMBER_3, XINE_EVENT_INPUT_NUMBER_4, XINE_EVENT_INPUT_NUMBER_5,
@@ -2449,10 +2436,10 @@ static void do_event(commands_t *cmd, client_info_t *client_info) {
 	};
 	  
 
-	if(((sscanf(arg, "%d", &value)) == 1) ||
-	   ((sscanf(arg, "+%d", &value)) == 1)) {
+	if(((sscanf(arg, "%zu", &value)) == 1) ||
+	   ((sscanf(arg, "+%zu", &value)) == 1)) {
 	  
-	  if((value >= 0) && (value < (sizeof(input_numbers) / sizeof(input_numbers[0]))))
+	  if(value < (sizeof(input_numbers) / sizeof(input_numbers[0])))
 	    xine_event.type = input_numbers[value];
 	}
       }
@@ -2551,14 +2538,14 @@ static void do_halt(commands_t *cmd, client_info_t *client_info) {
 }
 
 static void network_messenger(void *data, char *message) {
-  int socket = (int) data;
+  int socket = (int)(intptr_t) data;
   
-  sock_write(socket, message);
+  sock_write(socket, "%s", message);
 }
 
 static void do_snap(commands_t *cmd, client_info_t *client_info) {
   create_snapshot(gGui->mmk.mrl, 
-		  network_messenger, network_messenger, (void *)client_info->socket);
+		  network_messenger, network_messenger, (void *)(intptr_t)client_info->socket);
 }
 
 /*
@@ -2573,12 +2560,12 @@ static void say_hello(client_info_t *client_info) {
   struct hostent *hp = NULL;
   
   if(!gethostname(myfqdn, 255) && (hp = gethostbyname(myfqdn)) != NULL) {
-    sprintf(buf, "%s %s %s remote server. Nice to meet you.\n", hp->h_name, PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s %s\n", hp->h_name, PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
   else {
-    sprintf(buf, "%s %s remote server. Nice to meet you.\n", PACKAGE, VERSION);
+    snprintf(buf, sizeof(buf), "%s %s %s\n", PACKAGE, VERSION, "remote server. Nice to meet you.");
   }
-  sock_write(client_info->socket, buf);
+  sock_write(client_info->socket,"%s",  buf);
   
 }
 
@@ -2638,15 +2625,11 @@ static void parse_destock_remain(client_info_t *client_info) {
 	p++;
       
       if(p)
-	sprintf(remaining, "%s", (_atoa(p)));
+	strcpy(remaining, (_atoa(p)));
       
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(c) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * (strlen(c) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, sizeof(char *) * (strlen(c) + 1));
       
-      sprintf(client_info->command.line, "%s", c);
+      strcpy(client_info->command.line, c);
       
       if(p) {
 	/* remove last ';' */
@@ -2661,7 +2644,7 @@ static void parse_destock_remain(client_info_t *client_info) {
 	if(strlen(remaining)) {
 	  client_info->command.remain = (char *) realloc(client_info->command.remain, 
 							 sizeof(char *) * (strlen(remaining) + 1));
-	  sprintf(client_info->command.remain, "%s", remaining);
+	  strcpy(client_info->command.remain, remaining);
 	}
 	else {
 	  SAFE_FREE(client_info->command.remain);
@@ -2674,14 +2657,10 @@ static void parse_destock_remain(client_info_t *client_info) {
       
     }
     else { /* no ';' in remain, copy AS IS remain to line */
-      if(client_info->command.line)
-	client_info->command.line = (char *) realloc(client_info->command.line, 
-						     sizeof(char *) * (strlen(client_info->command.remain) + 1));
-      else
-	client_info->command.line = (char *) xine_xmalloc(sizeof(char *) * 
-							  (strlen(client_info->command.remain) + 1));
+      client_info->command.line = (char *) realloc(client_info->command.line, 
+						   sizeof(char *) * (strlen(client_info->command.remain) + 1));
       
-      sprintf(client_info->command.line, "%s", client_info->command.remain);
+      strcpy(client_info->command.line, client_info->command.remain);
       
       SAFE_FREE(client_info->command.remain);
     }
@@ -2730,15 +2709,12 @@ static void parse_command(client_info_t *client_info) {
     cmd = NULL;
 
   if(cmd) {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, 
-						      strlen(client_info->command.line) - strlen(cmd) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) - strlen(cmd) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, 
+						    strlen(client_info->command.line) - strlen(cmd) + 1);
 
     memset(client_info->command.command, 0, sizeof(client_info->command.command));
-    snprintf(client_info->command.command, 
-	     (strlen(client_info->command.line) - strlen(cmd))+1, "%s", client_info->command.line);
+    strlcpy(client_info->command.command, client_info->command.line,
+	    (strlen(client_info->command.line) - strlen(cmd)));
 
     cmd = _atoa(cmd);
     
@@ -2779,7 +2755,7 @@ static void parse_command(client_info_t *client_info) {
 	  if((*(pcmd - 1) != '\\') && (get_quote == 0)) {
 
 	  __end_args:
-	    sprintf(client_info->command.args[nargs], "%s", _atoa(buf));
+	    strcpy(client_info->command.args[nargs], _atoa(buf));
 	    nargs++;
 	    memset(&buf, 0, sizeof(buf));
 	    pb = buf;
@@ -2803,13 +2779,9 @@ static void parse_command(client_info_t *client_info) {
     }
   }
   else {
-    if(client_info->command.command)
-      client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
-    else
-      client_info->command.command = (char *) xine_xmalloc(strlen(client_info->command.line) + 1);
+    client_info->command.command = (char *) realloc(client_info->command.command, strlen(client_info->command.line) + 1);
     
-    memset(client_info->command.command, 0, sizeof(client_info->command.command));
-    sprintf(client_info->command.command, "%s", client_info->command.line);
+    strcpy(client_info->command.command, client_info->command.line);
   }
 
 #if 0
@@ -2896,7 +2868,7 @@ static void *client_thread(void *data) {
   memset(&client_info->passwd, 0, sizeof(client_info->passwd));
   
   for(i = 0; i < 256; i++)
-    client_info->command.args[i] = (char *) xine_xmalloc(sizeof(char *) * 2048);
+    client_info->command.args[i] = (char *) malloc(sizeof(char) * 2048);
 
   say_hello(client_info);
   
@@ -2933,14 +2905,22 @@ static void *server_thread(void *data) {
   char            *service;
   struct servent  *serv_ent;
   int              msock;
+  
+  if(gGui->network_port) {
+    int len = (int) log10(gGui->network_port) + 1;
 
-  /*  Search in /etc/services if a xinectl entry exist */
-  if((serv_ent = getservbyname("xinectl", "tcp")) != NULL) {
-    service = (char *) xine_xmalloc(ntohs(serv_ent->s_port));
-    sprintf(service, "%u", ntohs(serv_ent->s_port));
-  }
-  else
-    service = strdup(DEFAULT_XINECTL_PORT);
+    service = (char *) malloc(len + 1);
+    sprintf(service, "%u", gGui->network_port);
+  } 
+  else {
+    /*  Search in /etc/services if a xinectl entry exist */
+    if((serv_ent = getservbyname("xinectl", "tcp")) != NULL) {
+      service = (char *) malloc(ntohs(serv_ent->s_port));
+      sprintf(service, "%u", ntohs(serv_ent->s_port));
+    }
+    else
+      service = strdup(DEFAULT_XINECTL_PORT);
+  }  
   
   /* Load passwd file */
   /* password file syntax is:
@@ -2969,10 +2949,10 @@ static void *server_thread(void *data) {
    */
   {
     char        *passwdfile = ".xine/passwd";
-    char         passwdfilename[(strlen((get_homedir())) + strlen(passwdfile)) + 2];
+    char         passwdfilename[(strlen((xine_get_homedir())) + strlen(passwdfile)) + 2];
     struct stat  st;
     
-    sprintf(passwdfilename, "%s/%s", (get_homedir()), passwdfile);
+    snprintf(passwdfilename, sizeof(passwdfilename), "%s/%s", (xine_get_homedir()), passwdfile);
     
     if(stat(passwdfilename, &st) < 0) {
       fprintf(stderr, "ERROR: there is no password file for network access.!\n");
@@ -2988,15 +2968,15 @@ static void *server_thread(void *data) {
 
   while(gGui->running) {
     client_info_t       *client_info;
-    int                  lsin;
+    socklen_t            lsin;
     pthread_t            thread_client;
     
     msock = sock_serv(service, "tcp", 5);
     
-    client_info = (client_info_t *) xine_xmalloc(sizeof(client_info_t));
-    lsin = sizeof(client_info->sin);
+    client_info = (client_info_t *) calloc(1, sizeof(client_info_t));
+    lsin = sizeof(client_info->fsin.in);
     
-    client_info->socket = accept(msock, (struct sockaddr *)&(client_info->sin), &lsin);
+    client_info->socket = accept(msock, &(client_info->fsin.sa), &lsin);
     client_info->authentified = is_client_authorized(client_info);
     
     if(client_info->socket < 0) {
@@ -3044,41 +3024,6 @@ void start_remote_server(void) {
   
 }
 
-const char *get_homedir(void) {
-#ifdef HAVE_GETPWUID_R
-  struct passwd pwd;
-#endif
-  struct passwd *pw = NULL;
-  static char homedir[BUFSIZ] = {0,};
-
-  if(homedir[0])
-    return homedir;
-
-#ifdef HAVE_GETPWUID_R
-  if(getpwuid_r(getuid(), &pwd, homedir, sizeof(homedir), &pw) != 0 || pw == NULL) {
-#else
-  if((pw = getpwuid(getuid())) == NULL) {
-#endif
-    char *tmp = getenv("HOME");
-    if(tmp) {
-      strncpy(homedir, tmp, sizeof(homedir));
-      homedir[sizeof(homedir) - 1] = '\0';
-    }
-  } else {
-    char *s = strdup(pw->pw_dir);
-    strncpy(homedir, s, sizeof(homedir));
-    homedir[sizeof(homedir) - 1] = '\0';
-    free(s);
-  }
-
-  if(!homedir[0]) {
-    printf("Unable to get home directory, set it to /tmp.\n");
-    strcpy(homedir, "/tmp");
-  }
-
-  return homedir;
-}
-
 #endif
 
 #endif  /* HAVE_READLINE */
