@@ -13,15 +13,11 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id: mediamarks.c,v 1.6 2006/06/19 13:56:50 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
  *
  * parsing mediamarks
  */
 
-#define _GNU_SOURCE   /* this is needed for getline(..) and strndup(..) */
-
 #include "config.h"
 
 #include <stdio.h>
@@ -43,7 +39,6 @@
 #include "xine/xmlparser.h"
 #include "utils.h"
 
-extern gGui_t          *gGui;
 
 #define TYPE_NONE (0)
 #define TYPE_DIR  (1)
@@ -61,7 +56,7 @@ extern gGui_t          *gGui;
 #define MM_ACTION_PLAY (1)
 #define MM_ACTION_ADD (2)
 
-const char *mm_action_strings[] = { "Play", "Add to PL" };
+static const char *mm_action_strings[] = { "Play", "Add to PL" };
 
 typedef struct playitem_s playitem_t;
 
@@ -232,7 +227,6 @@ static void free_subtree(list_t *list, int i) {
 static playitem_t *playitem_load (xml_node_t *node) {
 
   playitem_t *play_item;
-  char *type;
   struct stat filestat;
 
   play_item = ho_new(playitem_t);
@@ -245,8 +239,8 @@ static playitem_t *playitem_load (xml_node_t *node) {
       printf ("mediamarks: title = %s\n", play_item->title);
 #endif
     } else if (!strcasecmp (node->name, "ref")) {
+      const char *type = xml_parser_get_property(node, "type");
       play_item->mrl = ho_strdup(xml_parser_get_property (node, "href"));
-      type = xml_parser_get_property(node, "type");
       if(type) {
 	if(!strcasecmp(type, "multiple")) {
 	  play_item->type = TYPE_MULTIPLE;
@@ -315,8 +309,8 @@ static int read_directory(oxine_t *oxine, const char *dir, list_t *list) {
     while ((entp = readdir(dirp))) {
 
       struct stat filestat;
-      char mrl[1024];
-      char title[256];
+      char *mrl;
+      char *title;
       int type = 0;
       
       if((!strcmp(entp->d_name, "."))||(!strcmp(entp->d_name, "..")))
@@ -326,22 +320,24 @@ static int read_directory(oxine_t *oxine, const char *dir, list_t *list) {
       if(entp->d_name[0] == '.')
         continue;
 #endif
-      
-      snprintf(mrl, 1023, "%s/%s", dir, entp->d_name);
+
+      asprintf(&mrl, "%s/%s", dir, entp->d_name);
       stat(mrl, &filestat);
-      if(S_ISDIR(filestat.st_mode)) {
-      type = TYPE_RDIR;
-      snprintf(title, 255, "[%s]", entp->d_name);	  
-      }else if (S_ISREG(filestat.st_mode)) {
-      strncpy(title, entp->d_name, 255);
-      type = TYPE_RREG;
-      }
+
       if(file_is_m3u(mrl)) {
-      type = TYPE_M3U;
-      snprintf(title, 255, "[%s]", entp->d_name);
+        asprintf(&title, "[%s]", entp->d_name);
+	type = TYPE_M3U;
+      } else if(S_ISDIR(filestat.st_mode)) {
+        asprintf(&title, "[%s]", entp->d_name);	  
+	type = TYPE_RDIR;
+      } else if (S_ISREG(filestat.st_mode)) {
+	title = strdup(entp->d_name);
+	type = TYPE_RREG;
       }
 
-      item = playitem_new(type, title, mrl, list_new());
+      item = playitem_new(type, NULL, NULL, list_new());
+      item->mrl = mrl;
+      item->title = title;
 #if 0
       printf("mrl   : %s\n", item->mrl);
       printf("title : %s\n", item->title);
