@@ -13,9 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id: otk.c,v 1.4 2005/02/20 01:16:40 miguelfreitas Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
  *
  * otk - the osd toolkit
  */
@@ -62,9 +60,6 @@ struct otk_widget_s {
 
   int            widget_type;
   
-  otk_t          *otk;
-  odk_t          *odk;
-
   int             x, y, w, h;
   int             focus;
   int             selectable;
@@ -72,6 +67,9 @@ struct otk_widget_s {
   int             needcalc; /* used by layout container */
   int             major; /* if parent otk_window_t should draw/destroy this widget set this to 0 */
 
+  otk_t          *otk;
+  odk_t          *odk;
+
   otk_widget_t   *win;
 
   void (*draw) (otk_widget_t *this);
@@ -154,8 +152,8 @@ struct otk_list_s {
   int              num_entries;
   int              last_selected;
 
-  char            *font;
   int              font_size;
+  char            *font;
 
   otk_list_cb_t    cb;
   void            *cb_data;
@@ -186,16 +184,17 @@ struct otk_s {
 
   g_list_t     *windows;
 
-  int           showing;
-
   otk_widget_t *focus_ptr;
 
+  int           showing;
+
   int           textcolor_win;
   int           textcolor_but;
   int           col_sl1, col_sl2, col_sl1f, col_sl2f;
 
-  char         *button_font;
   int           button_font_size;
+  char         *button_font;
+
   char         *label_font;
   int           label_color;
   int           label_font_size;
@@ -789,7 +788,7 @@ static void button_destroy(otk_widget_t *this) {
   if (!is_correct_widget(this, OTK_WIDGET_BUTTON)) return;
 
   if(button->type == OTK_BUTTON_PIXMAP) 
-    if(button->pixmap) free(button->pixmap);
+    free(button->pixmap);
 
   remove_widget_from_win(this);
   if (this->otk->focus_ptr == this)
@@ -1626,7 +1625,7 @@ static void selector_selected (otk_widget_t *this) {
 
 }
 
-otk_widget_t *otk_selector_grid_new (const char **items, int num,
+otk_widget_t *otk_selector_grid_new (const char *const *items, int num,
 			      otk_selector_cb_t cb,
 			      void *cb_data) {
   otk_selector_t *selector;
@@ -1655,7 +1654,7 @@ otk_widget_t *otk_selector_grid_new (const char **items, int num,
 }
 
 otk_widget_t *otk_selector_new(otk_widget_t *win,int x, int y, 
-                           int w, int h, const char **items, int num,
+                           int w, int h, const char *const *items, int num,
 			   otk_selector_cb_t cb,
 			   void *cb_data) {
   otk_selector_t *selector;
