@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2003 Stefan Holst
+ * Copyright (C) 2002-2008 Stefan Holst
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id: oxine.c,v 1.6 2005/02/15 20:09:32 mroi Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
  *
  * oxine main program
  */
@@ -40,11 +38,11 @@
 #include "oxine_event.h"
 #include "xine/xmlparser.h"
 #include "utils.h"
+#include "globals.h"
 
 #include "mediamarks.h"
 #include "playlist.h"
 
-extern gGui_t          *gGui;
 static oxine_t         *oxine_instance = NULL;
 
 
@@ -59,6 +57,7 @@ struct menuitem_s {
 
 
 static void main_menu_cb(void *data);
+#if 0
 static void media_stop_cb(void *data);
 static void media_info_cb(void *data);
 static void media_pause_cb(void *data, int i);
@@ -99,7 +98,6 @@ static void media_tracks_list_cb(void *data, void *entry) {
   odk_open_and_play(oxine->odk, mrl);
 }
 
-/*
 static void media_tracks_quit(void *data) {
   oxine_t *oxine = (oxine_t*) data;
 
@@ -110,7 +108,6 @@ static void media_tracks_quit(void *data) {
   oxine->pauseplay = NULL;
   otk_clear(oxine->otk); 
 }
-*/
 
 static void media_tracks_cb(void *data) {
   oxine_t *oxine = (oxine_t *) data;
@@ -136,7 +133,6 @@ static void media_tracks_cb(void *data) {
   otk_draw_all(oxine->otk);
 }
 
-#if 0
 static void info_button_time(void *data) {
   oxine_t *oxine = (oxine_t *)data;
   int ret, pos_time, length;
@@ -167,6 +163,7 @@ static void media_info_close_cb(void *data) {
   if (oxine->lines[2]) ho_free(oxine->lines[2]);
 }
 
+#if 0
 static void media_info_cb(void *data) {
   oxine_t *oxine = (oxine_t*) data;
   int pos_time, length, ret;
@@ -263,6 +260,7 @@ static void media_info_cb(void *data) {
 
   schedule_job(5000, media_info_close_cb, oxine);
 }
+#endif
 
 static void shutdown_cb (void *data) {
   gui_execute_action_id(ACTID_QUIT);
@@ -283,7 +281,7 @@ static void autoplay_cb (void *data) {
   char *parameter = (char *) data;
   oxine_t *oxine = oxine_instance;
   int    num_mrls, j;
-  char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
+  char **autoplay_mrls = xine_get_autoplay_mrls (__xineui_global_xine_instance,
                          parameter,
                          &num_mrls);
 
@@ -308,6 +306,7 @@ static void autoplay_cb (void *data) {
 }  
 
 
+#if 0
 static void playing_menu_update(void *data) {
   oxine_t *oxine = (oxine_t *) data;
   int pos_time, length;
@@ -319,7 +318,6 @@ static void playing_menu_update(void *data) {
     }
 }
 
-#if 0
 static void playing_menu_cb(void *data) {
    oxine_t *oxine = (oxine_t *) data;
    otk_widget_t *b, *l;
@@ -427,8 +425,7 @@ static char *read_entire_file (const char *mrl, int *file_size) {
 static menuitem_t *menuitem_load(xml_node_t *node) {
   
   menuitem_t *item = ho_new(menuitem_t);
-  char *type;
-  
+
   item->x = atoi(xml_parser_get_property(node, "x"));
   item->y = atoi(xml_parser_get_property(node, "y"));
   item->w = atoi(xml_parser_get_property(node, "width"));
@@ -437,38 +434,33 @@ static menuitem_t *menuitem_load(xml_node_t *node) {
   item->title = NULL;
   item->data = NULL;
 
-  node = node->child;
-  while( node ) {
+  if ( (node = node->child) == NULL )
+    return item;
 
+  do {
     if (!strcasecmp (node->name, "title")) {
       item->title = ho_strdup (node->data);
     } else if (!strcasecmp (node->name, "action")) {
+      const char *type = xml_parser_get_property(node, "type");
 
-      type = xml_parser_get_property(node, "type");
       if(type) {
         if(!strcasecmp(type, "autoplay")) {
           item->data = ho_strdup(xml_parser_get_property (node, "parameter"));
           item->func = autoplay_cb;
-        }
-        if(!strcasecmp(type, "mrl")) {
+        } else if(!strcasecmp(type, "mrl")) {
           item->data = ho_strdup(xml_parser_get_property (node, "parameter"));
           item->func = mrl_cb;
-        }
-        if(!strcasecmp(type, "mediamarks")) {
+        } else if(!strcasecmp(type, "mediamarks")) {
           item->func = mediamarks_cb;
-        }
-        if(!strcasecmp(type, "playlist")) {
+        } else if(!strcasecmp(type, "playlist")) {
           item->func = playlist_cb;
-        }
-        if(!strcasecmp(type, "shutdown")) {
+        } else if(!strcasecmp(type, "shutdown")) {
           item->func = shutdown_cb;
-        }
-        if(!strcasecmp(type, "shell")) {
+        } else if(!strcasecmp(type, "shell")) {
         }
       }
     }
-    node=node->next;
-  }
+  } while ( (node = node->next) != NULL );
 
   return item;
 }
@@ -534,7 +526,7 @@ static void main_menu_init(oxine_t *oxine)
   snprintf(mmpath,sizeof(mmpath),"%s/.xine/oxine/mainmenu", xine_get_homedir());
   if (!read_main_menu(oxine, oxine->main_menu_items, mmpath)) {
     lprintf("trying to load system wide mainmenu\n");
-    snprintf(mmpath,1024,"%s/mainmenu", XINE_OXINEDIR);
+    snprintf(mmpath, sizeof(mmpath),"%s/mainmenu", XINE_OXINEDIR);
     if (read_main_menu(oxine, oxine->main_menu_items, mmpath)) {
       /**/
     } else {
@@ -685,7 +677,7 @@ static oxine_t *create_oxine(void) {
     
   oxine->main_menu_cb = main_menu_cb;
 
-  oxine->xine = gGui->xine;
+  oxine->xine = __xineui_global_xine_instance;
   
   oxine->cd_mountpoint = 
   xine_config_register_string (oxine->xine,
