@@ -13,9 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id: utils.c,v 1.3 2006/06/19 13:56:50 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
  *
  * utilities
  */
@@ -337,8 +335,8 @@ void *_gen_free(void* content) {
     size_t        wSize=(char*)(prefix->postfix+1)-(char*)prefix;
     
     RemoveFromLinkedList( prefix );
+    free(prefix->tag);
     memset( prefix, 0, wSize );
-    if (prefix->tag) free(prefix->tag);
     free(prefix);
   }
 
@@ -497,10 +495,8 @@ static void RenderDesc(prefix_tag_t* prefix, char* lpBuffer) {
       sprintf( lpBuffer+strlen(lpBuffer), "%s:%ld ",
 	  prefix->filename, prefix->line );
     }
-    if (prefix->tag) {
-      sprintf( lpBuffer+strlen(lpBuffer), "%s",
-	  prefix->tag );
-    }
+    if (prefix->tag)
+      strcat(lpBuffer, prefix->tag);
   }
   else {
     strcpy( lpBuffer, "(bad pointer given)" );
