@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: panel.c,v 1.227 2006/06/29 14:08:13 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * xine panel related stuff
  *
@@ -31,13 +29,8 @@
 #include <string.h>
 #include <pthread.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
 #include "common.h"
 
-extern gGui_t     *gGui;
 
 _panel_t          *panel = NULL;
 
@@ -171,7 +164,7 @@ static void panel_exit(xitk_widget_t *w, void *data) {
     XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(panel->widget_list)));
     XUnlockDisplay(gGui->display);
 
-    free(panel->widget_list);
+    XITK_WIDGET_LIST_FREE(panel->widget_list);
     
     free(panel);
     panel = NULL;  
@@ -328,7 +321,7 @@ void panel_update_runtime_display(void) {
     }
     else
       if(remain < 0)
-	snprintf(timestr, sizeof(timestr), "%s", "  :  :  ");
+	strcpy(timestr, "  :  :  ");
       else {
 	if(rdays > 0)
 	  snprintf(timestr, sizeof(timestr), "% 4d::%02d", rdays, remain / (60*60));
@@ -338,7 +331,7 @@ void panel_update_runtime_display(void) {
     
   }
   else
-    snprintf(timestr, sizeof(timestr), "%s", "--:--:--");
+    strcpy(timestr, "--:--:--");
   
   if(length > 0) {
     int days;
@@ -432,10 +425,8 @@ static void *slider_loop(void *dummy) {
 	  if(gGui->playlist.num && gGui->playlist.cur >= 0 && gGui->playlist.mmk &&
 	     gGui->playlist.mmk[gGui->playlist.cur]) {
 	    if(strcmp(gGui->mmk.ident, ident)) {
-	      if(gGui->mmk.ident)
-		free(gGui->mmk.ident);
-	      if(gGui->playlist.mmk[gGui->playlist.cur]->ident)
-		free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
+	      free(gGui->mmk.ident);
+	      free(gGui->playlist.mmk[gGui->playlist.cur]->ident);
 	      
 	      gGui->mmk.ident = strdup(ident);
 	      gGui->playlist.mmk[gGui->playlist.cur]->ident = strdup(ident);
@@ -954,7 +945,7 @@ void panel_add_autoplay_buttons(void) {
   int                        x, y, dir;
   int                        i = 0, max;
   xitk_labelbutton_widget_t  lb;
-  const char *const         *autoplay_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
+  const char *const         *autoplay_plugins = xine_get_autoplay_input_plugin_ids(__xineui_global_xine_instance);
   const char                *autoplay_label;
   
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
@@ -979,7 +970,7 @@ void panel_add_autoplay_buttons(void) {
 				 xitk_labelbutton_create (panel->widget_list, 
 							  gGui->skin_config, &lb)));
       xitk_set_widget_tips_and_timeout(panel->autoplay_plugins[i],
-				       (char *) xine_get_input_plugin_description(gGui->xine, autoplay_label),
+				       (char *) xine_get_input_plugin_description(__xineui_global_xine_instance, autoplay_label),
 				       panel->tips.timeout);
       
       if(!panel->tips.enable)
@@ -1085,7 +1076,7 @@ void panel_init (void) {
   XSizeHints                hint;
   XWMHints                 *wm_hint;
   XSetWindowAttributes      attr;
-  char                     *title;
+  char                     *title = _("xine Panel");
   Atom                      prop;
   MWMHints                  mwmhints;
   XClassHint               *xclasshint;
@@ -1095,14 +1086,12 @@ void panel_init (void) {
   xitk_slider_widget_t      sl;
   xitk_widget_t            *w;
   
-  xine_strdupa(title, _("xine Panel"));
-
   XITK_WIDGET_INIT(&b, gGui->imlib_data);
   XITK_WIDGET_INIT(&cb, gGui->imlib_data);
   XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
   XITK_WIDGET_INIT(&sl, gGui->imlib_data);
 
-  panel = (_panel_t *) xine_xmalloc(sizeof(_panel_t));
+  panel = (_panel_t *) calloc(1, sizeof(_panel_t));
 
   panel->playback_widgets.enabled = -1;
 
@@ -1126,14 +1115,14 @@ void panel_init (void) {
    * open the panel window
    */
 
-  panel->x = hint.x = xine_config_register_num (gGui->xine, "gui.panel_x", 
+  panel->x = hint.x = xine_config_register_num (__xineui_global_xine_instance, "gui.panel_x", 
 						200,
 						CONFIG_NO_DESC,
 						CONFIG_NO_HELP,
 						CONFIG_LEVEL_DEB,
 						CONFIG_NO_CB,
 						CONFIG_NO_DATA);
-  panel->y = hint.y = xine_config_register_num (gGui->xine, "gui.panel_y",
+  panel->y = hint.y = xine_config_register_num (__xineui_global_xine_instance, "gui.panel_y",
 						100,
 						CONFIG_NO_DESC,
 						CONFIG_NO_HELP,
@@ -1180,10 +1169,21 @@ void panel_init (void) {
   XSelectInput(gGui->display, gGui->panel_window, INPUT_MOTION | KeymapStateMask);
   XUnlockDisplay(gGui->display);
   
-  if(!video_window_is_visible())
+  /*
+   * The following is more or less a hack to keep the panel window visible
+   * with and without focus in windowed and fullscreen mode.
+   * Different WMs and even different versions behave different.
+   * There is no guarantee that it works with all WMs/versions.
+   */
+  if(!video_window_is_visible()) {
+    if(!(xitk_get_wm_type() & WM_TYPE_KWIN))
+      xitk_unset_wm_window_type(gGui->panel_window, WINDOW_TYPE_TOOLBAR);
     xitk_set_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
-  else
+  } else {
     xitk_unset_wm_window_type(gGui->panel_window, WINDOW_TYPE_NORMAL);
+    if(!(xitk_get_wm_type() & WM_TYPE_KWIN))
+      xitk_set_wm_window_type(gGui->panel_window, WINDOW_TYPE_TOOLBAR);
+  }
   
   if(is_layer_above())
     xitk_set_layer_above(gGui->panel_window);
@@ -1490,14 +1490,14 @@ void panel_init (void) {
   xitk_set_widget_tips(w, _("Open Location"));
   
 
-  panel->tips.enable = xine_config_register_bool(gGui->xine, "gui.tips_visible", 
+  panel->tips.enable = xine_config_register_bool(__xineui_global_xine_instance, "gui.tips_visible", 
 						 1,
 						 _("gui tips visibility"), 
 						 _("If disabled, no tooltips are shown."), 
 						 CONFIG_LEVEL_ADV,
 						 panel_enable_tips_cb,
 						 CONFIG_NO_DATA);
-  panel->tips.timeout = (unsigned long) xine_config_register_num(gGui->xine, "gui.tips_timeout", 
+  panel->tips.timeout = (unsigned long) xine_config_register_num(__xineui_global_xine_instance, "gui.tips_timeout", 
 								 5000,
 								 _("Tips timeout (ms)"), 
 								 _("Persistence time of tooltips, in milliseconds."), 
@@ -1512,7 +1512,7 @@ void panel_init (void) {
    * show panel 
    */
   {
-    int visible = xine_config_register_bool(gGui->xine, "gui.panel_visible", 
+    int visible = xine_config_register_bool(__xineui_global_xine_instance, "gui.panel_visible", 
 					    1,
 					    _("gui panel visibility"),
 					    CONFIG_NO_HELP,
