@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: browser.c,v 1.64 2004/12/15 21:10:47 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -147,7 +145,7 @@ static void notify_destroy(xitk_widget_t *w) {
 
     private_data = (browser_private_data_t *) w->private_data;
     
-    for(i = WBSTART; i < private_data->max_length + WBSTART; i++)
+    for(i = WBSTART; i < private_data->max_length + WBSTART + EXTRA_BTNS; i++)
       XITK_FREE(private_data->bt[i]);
     
     XITK_FREE(private_data->skin_element_name);
