@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2008 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: widget.h,v 1.36 2004/07/21 19:56:17 f1rmb Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  */
 
@@ -119,6 +117,7 @@ struct xitk_widget_list_s {
 
   Window                      win;
   GC                          gc;
+  int                         destroy;
 };
 
 /* ****************************************************************** */
